@@ -244,7 +244,7 @@ void ptNormalize(poly* gStar, const number p, const ring r)
 
 void ptNormalize(ideal I, const number p, const ring r)
 {
-  for (int i=0; i<idSize(I); i++)
+  for (int i=0; i<IDELEMS(I); i++)
     ptNormalize(&(I->m[i]),p,r);
   return;
 }
@@ -273,44 +273,18 @@ BOOLEAN ptNormalize(leftv res, leftv args)
 }
 #endif //NDEBUG
 
-#ifndef NDEBUG
-BOOLEAN pReduceDebug(leftv res, leftv args)
-{
-  leftv u = args;
-  if ((u != NULL) && (u->Typ() == POLY_CMD))
-  {
-    poly g; number p = n_Init(3,currRing->cf);
-    omUpdateInfo();
-    Print("usedBytesBefore=%ld\n",om_Info.UsedBytes);
-    g = (poly) u->CopyD();
-    (void) pReduce(g,p,currRing);
-    p_Delete(&g,currRing);
-    omUpdateInfo();
-    Print("usedBytesAfter=%ld\n",om_Info.UsedBytes);
-    g = (poly) u->CopyD();
-    (void) pReduce(g,p,currRing);
-    n_Delete(&p,currRing->cf);
-    res->rtyp = POLY_CMD;
-    res->data = (char*) g;
-    return FALSE;
-  }
-  return TRUE;
-}
-#endif //NDEBUG
-
 void pReduce(ideal &I, const number p, const ring r)
 {
-  int k = idSize(I);
+  int k = IDELEMS(I);
   for (int i=0; i<k; i++)
   {
     if (I->m[i]!=NULL)
     {
       number c = p_GetCoeff(I->m[i],r);
-      if (!n_Equal(p,c,r->cf))
+      if (!n_DivBy(p,c,r->cf))
         pReduce(I->m[i],p,r);
     }
   }
-  return;
 }
 
 
@@ -354,39 +328,6 @@ bool ppreduceInitially(poly* hStar, const poly g, const ring r)
 }
 
 
-#ifndef NDEBUG
-BOOLEAN ppreduceInitially0(leftv res, leftv args)
-{
-  leftv u = args;
-  if ((u != NULL) && (u->Typ() == POLY_CMD))
-  {
-    leftv v = u->next;
-    if ((v != NULL) && (v->Typ() == POLY_CMD))
-    {
-      poly g,h;
-      omUpdateInfo();
-      Print("usedBytesBefore=%ld\n",om_Info.UsedBytes);
-      h = (poly) u->CopyD();
-      g = (poly) v->CopyD();
-      (void)ppreduceInitially(&h,g,currRing);
-      p_Delete(&h,currRing);
-      p_Delete(&g,currRing);
-      omUpdateInfo();
-      Print("usedBytesAfter=%ld\n",om_Info.UsedBytes);
-      h = (poly) u->CopyD();
-      g = (poly) v->CopyD();
-      (void)ppreduceInitially(&h,g,currRing);
-      p_Delete(&g,currRing);
-      res->rtyp = POLY_CMD;
-      res->data = (char*) h;
-      return FALSE;
-    }
-  }
-  return TRUE;
-}
-#endif //NDEBUG
-
-
 /***
  * reduces I initially with respect to itself and with respect to p-t.
  * also sorts the generators of I with respect to the leading monomials in descending order.
@@ -394,13 +335,14 @@ BOOLEAN ppreduceInitially0(leftv res, leftv args)
  **/
 bool ppreduceInitially(ideal I, const number p, const ring r)
 {
-  int m=idSize(I),n=m; poly cache;
+  idSkipZeroes(I);
+  int m=IDELEMS(I),n=m; poly cache;
   do
   {
     int j=0;
     for (int i=1; i<n; i++)
     {
-      if (p_LmCmp(I->m[i-1],I->m[i],r)<0)
+      if (p_LmCmp(I->m[i-1],I->m[i],r)<0) /*p_LmCmp(p,q): requires: p,q!=NULL*/
       {
         cache=I->m[i-1];
         I->m[i-1]=I->m[i];
@@ -437,39 +379,6 @@ bool ppreduceInitially(ideal I, const number p, const ring r)
 }
 
 
-#ifndef NDEBUG
-BOOLEAN ppreduceInitially1(leftv res, leftv args)
-{
-  leftv u = args;
-  if ((u != NULL) && (u->Typ() == IDEAL_CMD))
-  {
-    leftv v = u->next;
-    if ((v != NULL) && (v->Typ() == NUMBER_CMD))
-    {
-      ideal I; number p;
-      omUpdateInfo();
-      Print("usedBytesBefore=%ld\n",om_Info.UsedBytes);
-      I = (ideal) u->CopyD();
-      p = (number) v->CopyD();
-      (void) ppreduceInitially(I,p,currRing);
-      id_Delete(&I,currRing);
-      n_Delete(&p,currRing->cf);
-      omUpdateInfo();
-      Print("usedBytesAfter=%ld\n",om_Info.UsedBytes);
-      I = (ideal) u->CopyD();
-      p = (number) v->CopyD();
-      (void) ppreduceInitially(I,p,currRing);
-      n_Delete(&p,currRing->cf);
-      res->rtyp = IDEAL_CMD;
-      res->data = (char*) I;
-      return FALSE;
-    }
-  }
-  return TRUE;
-}
-#endif //NDEBUG
-
-
 /***
  * inserts g into I and reduces I with respect to itself and p-t
  * returns the position in I in which g was inserted
@@ -480,11 +389,12 @@ int ppreduceInitially(ideal I, const number p, const poly g, const ring r)
   id_Test(I,r);
   p_Test(g,r);
   idInsertPoly(I,g);
-  int n=idSize(I);
+  idSkipZeroes(I);
+  int n=IDELEMS(I);
   int j;
   for (j=n-1; j>0; j--)
   {
-    if (p_LmCmp(I->m[j], I->m[j-1],r)>0)
+    if (p_LmCmp(I->m[j], I->m[j-1],r)>0) /*p_LmCmp(p,q) requires: p,q!=NULL */
     {
       poly cache = I->m[j];
       I->m[j] = I->m[j-1];
@@ -532,45 +442,6 @@ int ppreduceInitially(ideal I, const number p, const poly g, const ring r)
 }
 
 
-#ifndef NDEBUG
-BOOLEAN ppreduceInitially2(leftv res, leftv args)
-{
-  leftv u = args;
-  if ((u != NULL) && (u->Typ() == IDEAL_CMD))
-  {
-    leftv v = u->next;
-    if ((v != NULL) && (v->Typ() == NUMBER_CMD))
-    {
-      leftv w = v->next;
-      if ((w != NULL) && (w->Typ() == POLY_CMD))
-      {
-        ideal I; number p; poly g;
-        omUpdateInfo();
-        Print("usedBytesBefore=%ld\n",om_Info.UsedBytes);
-        I = (ideal) u->CopyD();
-        p = (number) v->CopyD();
-        g = (poly) w->CopyD();
-        (void) ppreduceInitially(I,p,g,currRing);
-        id_Delete(&I,currRing);
-        n_Delete(&p,currRing->cf);
-        omUpdateInfo();
-        Print("usedBytesAfter=%ld\n",om_Info.UsedBytes);
-        I = (ideal) u->CopyD();
-        p = (number) v->CopyD();
-        g = (poly) w->CopyD();
-        (void) ppreduceInitially(I,p,g,currRing);
-        n_Delete(&p,currRing->cf);
-        res->rtyp = IDEAL_CMD;
-        res->data = (char*) I;
-        return FALSE;
-      }
-    }
-  }
-  return TRUE;
-}
-#endif //NDEBUG
-
-
 static poly ppNext(poly p, int l)
 {
   poly q = p;
@@ -665,26 +536,35 @@ bool ppreduceInitially(ideal &H, const number p, const ideal G, const ring r)
    *   term j and subsequent terms need to be checked for reduction.
    *   T is sorted by the ordering on the temrs the pairs correspond to.
    **/
-  int m=idSize(H);
+  int m=IDELEMS(H);
   ideal I = idInit(m);
   std::vector<mark> T;
   for (int i=0; i<m; i++)
   {
-    I->m[i]=H->m[i];
-    if (pNext(I->m[i])!=NULL)
-      T.push_back(std::pair<int,int>(i,1));
+    if(H->m[i]!=NULL)
+    {
+      I->m[i]=H->m[i];
+      if (pNext(I->m[i])!=NULL)
+        T.push_back(std::pair<int,int>(i,1));
+    }
   }
 
   /***
    * Step 3: as long as the working list is not empty, successively reduce terms in it
    *   by adding suitable elements to I and reducing it initially with respect to itself
    **/
-  int k=idSize(G);
+  int k=IDELEMS(G);
   while (T.size()>0)
   {
     sortMarks(I,r,T);
-    int i=0; for (; i<k; i++)
-      if (p_LeadmonomDivisibleBy(G->m[i],getTerm(I,T[0]),r)) break;
+    int i=0;
+    for (; i<k; i++)
+    {
+      if(G->m[i]!=NULL)
+      {
+        if (p_LeadmonomDivisibleBy(G->m[i],getTerm(I,T[0]),r)) break;
+      }
+    }
     if (i<k)
     {
       poly g = p_One(r); poly h0 = getTerm(I,T[0]);
@@ -704,15 +584,19 @@ bool ppreduceInitially(ideal &H, const number p, const ideal G, const ring r)
   /***
    * Step 4: cleanup, delete all polynomials in I which have been added in Step 3
    **/
-  k=idSize(I);
+  k=IDELEMS(I);
   for (int i=0; i<k; i++)
   {
-    for (int j=0; j<m; j++)
+    if(I->m[i]!=NULL)
     {
-      if (p_LeadmonomDivisibleBy(H->m[j],I->m[i],r))
+      for (int j=0; j<m; j++)
       {
-        I->m[i]=NULL;
-        break;
+        if ((H->m[j]!=NULL)
+        && (p_LeadmonomDivisibleBy(H->m[j],I->m[i],r)))
+        {
+          I->m[i]=NULL;
+          break;
+        }
       }
     }
   }
@@ -721,37 +605,6 @@ bool ppreduceInitially(ideal &H, const number p, const ideal G, const ring r)
 }
 
 
-#ifndef NDEBUG
-BOOLEAN ppreduceInitially3(leftv res, leftv args)
-{
-  leftv u = args;
-  if ((u != NULL) && (u->Typ() == IDEAL_CMD))
-  {
-    leftv v = u->next;
-    if ((v != NULL) && (v->Typ() == NUMBER_CMD))
-    {
-      leftv w = v->next;
-      if ((w != NULL) && (w->Typ() == IDEAL_CMD))
-      {
-        ideal H,G; number p;
-        omUpdateInfo();
-        Print("usedBytesBefore=%ld\n",om_Info.UsedBytes);
-        H = (ideal) u->CopyD();
-        p = (number) v->CopyD();
-        G = (ideal) w->CopyD();
-        (void) ppreduceInitially(H,p,G,currRing);
-        n_Delete(&p,currRing->cf);
-        id_Delete(&G,currRing);
-        res->rtyp = IDEAL_CMD;
-        res->data = (char*) H;
-        return FALSE;
-      }
-    }
-  }
-  return TRUE;
-}
-#endif //NDEBUG
-
 /**
  * reduces I initially with respect to itself.
  * assumes that the generators of I are homogeneous in x and that p-t is in I.
@@ -764,21 +617,24 @@ bool ppreduceInitially(ideal I, const ring r, const number p)
    * Step 1: split up I into components of same degree in x
    *  the lowest component should only contain p-t
    **/
-  std::map<long,ideal> H; int n = idSize(I);
+  std::map<long,ideal> H; int n = IDELEMS(I);
   for (int i=0; i<n; i++)
   {
-    I->m[i] = p_Cleardenom(I->m[i],r);
-    long d = 0;
-    for (int j=2; j<=r->N; j++)
-      d += p_GetExp(I->m[i],j,r);
-    std::map<long,ideal>::iterator it = H.find(d);
-    if (it != H.end())
-      idInsertPoly(it->second,I->m[i]);
-    else
+    if(I->m[i]!=NULL)
     {
-      std::pair<long,ideal> Hd(d,idInit(1));
-      Hd.second->m[0] = I->m[i];
-      H.insert(Hd);
+      I->m[i] = p_Cleardenom(I->m[i],r);
+      long d = 0;
+      for (int j=2; j<=r->N; j++)
+        d += p_GetExp(I->m[i],j,r);
+      std::map<long,ideal>::iterator it = H.find(d);
+      if (it != H.end())
+        idInsertPoly(it->second,I->m[i]);
+      else
+      {
+        std::pair<long,ideal> Hd(d,idInit(1));
+        Hd.second->m[0] = I->m[i];
+        H.insert(Hd);
+      }
     }
   }
 
@@ -793,6 +649,7 @@ bool ppreduceInitially(ideal I, const ring r, const number p)
    *  and all lower components
    **/
   if (ppreduceInitially(Hi,p,r)) return true;
+  idSkipZeroes(Hi);
   id_Test(Hi,r);
   id_Test(I,r);
 
@@ -802,7 +659,7 @@ bool ppreduceInitially(ideal I, const ring r, const number p)
 
   for (it++; it!=H.end(); it++)
   {
-    int l=idSize(Hi); int k=l; poly cache;
+    int l=IDELEMS(Hi); int k=l; poly cache;
     /**
      * sorts Hi according to degree in t in descending order
      * (lowest first, highest last)
@@ -854,30 +711,6 @@ bool ppreduceInitially(ideal I, const ring r, const number p)
 }
 
 
-#ifndef NDEBUG
-BOOLEAN reduceInitiallyDebug(leftv res, leftv args)
-{
-  leftv u = args;
-  if ((u != NULL) && (u->Typ() == IDEAL_CMD))
-  {
-    leftv v = u->next;
-    if ((v != NULL) && (v->Typ() == NUMBER_CMD))
-    {
-      omUpdateInfo();
-      Print("usedBytesBefore=%ld\n",om_Info.UsedBytes);
-      ideal I = (ideal) u->CopyD();
-      number p = (number) v->Data();
-      (void) ppreduceInitially(I,currRing,p);
-      res->rtyp = IDEAL_CMD;
-      res->data = (char*) I;
-      return FALSE;
-    }
-  }
-  return TRUE;
-}
-#endif
-
-
 // BOOLEAN ppreduceInitially(leftv res, leftv args)
 // {
 //   leftv u = args;
