@@ -110,8 +110,6 @@ void feHelp(char *str)
 
   BOOLEAN key_is_regexp = (strchr(str, '*') != NULL);
 
-  // try proc help and library help
-  if (! key_is_regexp && heOnlineHelp(str)) return;
 
   heEntry_s hentry;
   memset(&hentry,0,sizeof(hentry));
@@ -133,9 +131,13 @@ void feHelp(char *str)
     StringSetS("");
     int found = heReKey2Entry(idxfile, str, &hentry);
 
-    // Try to match with str*
+
     if (found == 0)
     {
+      // try proc help and library help
+      if (! key_is_regexp && heOnlineHelp(str)) return;
+
+      // Try to match with str*
       char mkey[MAX_HE_ENTRY_LENGTH];
       strcpy(mkey, str);
       strcat(mkey, "*");
@@ -702,7 +704,7 @@ static BOOLEAN heOnlineHelp(char* s)
     return FALSE;
   }
 
-  char libnamebuf[128];
+  char libnamebuf[1024];
   FILE *fp=NULL;
   // first, search for library of that name
   if ((str[1]!='\0') &&
@@ -908,10 +910,16 @@ static void heGenHelp(heEntry hentry, int br)
                    #ifdef HAVE_VSNPRINTF
                    {
                      if (*p=='H')
+                     #ifdef SINGULAR_4_2
+                       snprintf(temp,256,"%s/%d-%d/%s", htmldir,
+                                  SINGULAR_VERSION/1000,
+                                 (SINGULAR_VERSION % 1000)/100,
+                     #else
                        snprintf(temp,256,"%s/%d-%d-%d/%s", htmldir,
                                   SINGULAR_VERSION/1000,
                                  (SINGULAR_VERSION % 1000)/100,
                                  (SINGULAR_VERSION % 100)/10,
+                     #endif
                        hentry->url);
                      else
                        snprintf(temp,256,"%s/%s", htmldir, hentry->url);
@@ -1018,7 +1026,7 @@ static BOOLEAN heDummyInit(int /*warn*/, int /*br*/)
 }
 static void heDummyHelp(heEntry /*hentry*/, int /*br*/)
 {
-  Werror("No functioning help browser available.");
+  WerrorS("No functioning help browser available.");
 }
 
 static BOOLEAN heEmacsInit(int /*warn*/, int /*br*/)
