@@ -44,7 +44,7 @@ FILE *File_Profiling=NULL;
 // this is an upper limit for the size of monomials/numbers read via the interpreter
 #define MAX_FILE_BUFFER 4*4096
 // static long feBufferLength=INITIAL_PRINT_BUFFER;
-static char * feBuffer=(char *)omAlloc(INITIAL_PRINT_BUFFER);
+//static char * feBuffer=(char *)omAlloc(INITIAL_PRINT_BUFFER);
 
 /**************************************************************************
 * handling of 'voices'
@@ -62,7 +62,7 @@ Voice  *currentVoice = NULL;
 /*2
 * the name of the current 'Voice': the procname (or filename)
 */
-const char * sNoName_fe="_";
+const char sNoName_fe[]="_";
 const char * VoiceName()
 {
   if ((currentVoice!=NULL)
@@ -375,7 +375,9 @@ BOOLEAN exitVoice()
       omFree((ADDRESS)currentVoice->buffer);
       currentVoice->buffer=NULL;
     }
-    if ((currentVoice->prev==NULL)&&(currentVoice->sw==BI_file))
+    if ((currentVoice->prev==NULL)
+    &&(currentVoice->sw==BI_file)
+    &&(currentVoice->files!=stdin))
     {
       currentVoice->prev=feInitStdin(currentVoice);
     }
@@ -595,7 +597,13 @@ int feReadLine(char* b, int l)
       fseek(currentVoice->files,currentVoice->ftellptr,SEEK_SET);
       s=fgets(currentVoice->buffer+offset,(MAX_FILE_BUFFER-1-sizeof(ADDRESS))-offset,
               currentVoice->files);
-      if (s!=NULL) currentVoice->ftellptr=ftell(currentVoice->files);
+      if (s!=NULL)
+      {
+        currentVoice->ftellptr=ftell(currentVoice->files);
+        // ftell returns -1 for non-seekable streams, such as pipes
+        if (currentVoice->ftellptr<0)
+          currentVoice->ftellptr=0;
+      }
     }
     //else /* BI_buffer */ s==NULL  => return 0
     // done by the default return
@@ -663,11 +671,17 @@ Voice * feInitStdin(Voice *pp)
   Voice *p = new Voice;
   p->files = stdin;
   p->sw = (isatty(STDIN_FILENO)) ? BI_stdin : BI_file;
-  if ((pp!=NULL) && (pp->files==stdin))
+  if ((pp!=NULL) && (pp->sw==BI_stdin) && (pp->files==stdin))
   {
     p->files=freopen("/dev/tty","r",stdin);
     //stdin=p->files;
-    p->sw = BI_stdin;
+    if (p->files==NULL)
+    {
+      p->files = stdin;
+      p->sw = BI_file;
+    }
+    else
+      p->sw = BI_stdin;
   }
   p->filename   = omStrDup("STDIN");
   p->start_lineno   = 1;
