@@ -40,8 +40,12 @@ int produce_convert_table=0;
 #define ALLOW_ZERODIVISOR  0
 #define ZERODIVISOR_MASK 8
 
+#define ALLOW_ZZ (ALLOW_RING|NO_ZERODIVISOR)
+
 // bit 4 for warning, if used at toplevel
 #define WARN_RING        16
+// bit 5: do no try automatic conversions
+#define NO_CONVERSION    32
 
 /*=============== types =====================*/
 struct _scmdnames
@@ -141,9 +145,6 @@ const char * Tok2Cmdname(int tok)
   //if (tok==OBJECT) return "object";
   //if (tok==PRINT_EXPR) return "print_expr";
   if (tok==IDHDL) return "identifier";
-  #ifdef SINGULAR_4_1
-  if (tok==CRING_CMD) return "(c)ring";
-  #endif
   // we do not blackbox objects during table generation:
   //if (tok>MAX_TOK) return getBlackboxName(tok);
   int i = 0;
@@ -931,7 +932,7 @@ void ttGen4()
 }
 /*-------------------------------------------------------------------*/
 
-int main(int argc, char** argv)
+int main(int argc, char**)
 {
   if (argc>1)
   {
