@@ -37,7 +37,7 @@
 #include <Singular/blackbox.h>
 #include <Singular/number2.h>
 
-#ifdef SINGULAR_4_1
+#ifdef SINGULAR_4_2
 #include <Singular/number2.h>
 #endif
 #ifdef HAVE_DYNAMIC_LOADING
@@ -60,10 +60,9 @@ proclevel *procstack=NULL;
 
 idhdl currPackHdl = NULL;
 idhdl basePackHdl = NULL;
-package currPack =NULL;
-package basePack =NULL;
+package currPack = NULL;
+package basePack = NULL;
 idhdl currRingHdl = NULL;
-const char* iiNoName="_";
 
 void paCleanUp(package pack);
 
@@ -134,9 +133,11 @@ void *idrecDataInit(int t)
   switch (t)
   {
     //the type with init routines:
-#ifdef SINGULAR_4_1
+#ifdef SINGULAR_4_2
     case CNUMBER_CMD:
       return (void*)n2Init(0,NULL);
+    case CPOLY_CMD:
+      return (void*)p2Init(0,NULL);
     case CMATRIX_CMD:
 #endif
     case BIGINTMAT_CMD:
@@ -170,7 +171,7 @@ void *idrecDataInit(int t)
     case LINK_CMD:
       return (void*) omAlloc0Bin(sip_link_bin);
     case RING_CMD:
-      return (void*) omAlloc0Bin(sip_sring_bin);
+      return NULL;
     case PACKAGE_CMD:
     {
       package pa=(package)omAlloc0Bin(sip_package_bin);
@@ -188,9 +189,7 @@ void *idrecDataInit(int t)
     case RESOLUTION_CMD:
       return  (void *)omAlloc0(sizeof(ssyStrategy));
     //other types: without init (int,script,poly,def,package)
-    #ifdef SINGULAR_4_1
     case CRING_CMD:
-    #endif
     case INT_CMD:
     case DEF_CMD:
     case POLY_CMD:
@@ -277,15 +276,21 @@ idhdl enterid(const char * s, int lev, int t, idhdl* root, BOOLEAN init, BOOLEAN
     {
       if ((IDTYP(h) == t)||(t==DEF_CMD))
       {
-        if ((IDTYP(h)==PACKAGE_CMD)
-        && (strcmp(s,"Top")==0))
+        if (IDTYP(h)==PACKAGE_CMD)
         {
-          goto errlabel;
+          if (strcmp(s,"Top")==0)
+          {
+            goto errlabel;
+          }
+          else return *root;
+        }
+        else
+        {
+          if (BVERBOSE(V_REDEFINE))
+            Warn("redefining %s (%s)",s,my_yylinebuf);
+          if (s==IDID(h)) IDID(h)=NULL;
+          killhdl2(h,root,currRing);
         }
-        if (BVERBOSE(V_REDEFINE))
-          Warn("redefining %s **",s);
-        if (s==IDID(h)) IDID(h)=NULL;
-        killhdl2(h,root,currRing);
       }
       else
         goto errlabel;
@@ -301,7 +306,7 @@ idhdl enterid(const char * s, int lev, int t, idhdl* root, BOOLEAN init, BOOLEAN
         if ((IDTYP(h) == t)||(t==DEF_CMD))
         {
           if (BVERBOSE(V_REDEFINE))
-            Warn("redefining %s **",s);
+            Warn("redefining %s (%s)",s,my_yylinebuf);
           if (s==IDID(h)) IDID(h)=NULL;
           killhdl2(h,&currRing->idroot,currRing);
         }
@@ -320,7 +325,7 @@ idhdl enterid(const char * s, int lev, int t, idhdl* root, BOOLEAN init, BOOLEAN
         if ((IDTYP(h) == t)||(t==DEF_CMD))
         {
           if (BVERBOSE(V_REDEFINE))
-            Warn("redefining `%s` **",s);
+            Warn("redefining %s (%s)",s,my_yylinebuf);
           if (s==IDID(h)) IDID(h)=NULL;
           killhdl2(h,&IDROOT,NULL);
         }
@@ -366,13 +371,13 @@ void killid(const char * id, idhdl * ih)
     killhdl2(h,ih,currRing);
   }
   else
-    Werror("kill what ?");
+    WerrorS("kill what ?");
 }
 
 void killhdl(idhdl h, package proot)
 {
   int t=IDTYP(h);
-  if (((BEGIN_RING<t) && (t<END_RING) && (t!=QRING_CMD))
+  if (((BEGIN_RING<t) && (t<END_RING))
   || ((t==LIST_CMD) && (lRingDependend((lists)IDDATA(h)))))
     killhdl2(h,&currRing->idroot,currRing);
   else
@@ -449,7 +454,7 @@ void killhdl2(idhdl h, idhdl * ih, ring r)
     if (currPackHdl==h) currPackHdl=packFindHdl(currPack);
     iiCheckPack(currPack);
   }
-  else if ((IDTYP(h)==RING_CMD)||(IDTYP(h)==QRING_CMD))
+  else if (IDTYP(h)==RING_CMD)
     rKill(h);
   else if (IDDATA(h)!=NULL)
     s_internalDelete(IDTYP(h),IDDATA(h),r);
@@ -487,6 +492,7 @@ void killhdl2(idhdl h, idhdl * ih, ring r)
   omFreeBin((ADDRESS)h, idrec_bin);
 }
 
+#if 0
 idhdl ggetid(const char *n, BOOLEAN /*local*/, idhdl *packhdl)
 {
   idhdl h = IDROOT->get(n,myynest);
@@ -499,17 +505,17 @@ idhdl ggetid(const char *n, BOOLEAN /*local*/, idhdl *packhdl)
   if (h2==NULL) return h;
   return h2;
 }
+#endif
 
 idhdl ggetid(const char *n)
 {
   idhdl h = IDROOT->get(n,myynest);
   if ((h!=NULL)&&(IDLEV(h)==myynest)) return h;
-  idhdl h2=NULL;
   if (currRing!=NULL)
   {
-    h2 = currRing->idroot->get(n,myynest);
+    idhdl h2 = currRing->idroot->get(n,myynest);
+    if (h2!=NULL) return h2;
   }
-  if (h2!=NULL) return h2;
   if (h!=NULL) return h;
   if (basePack!=currPack)
     return basePack->idroot->get(n,myynest);
@@ -605,7 +611,7 @@ void  ipMoveId(idhdl tomove)
 {
   if ((currRing!=NULL)&&(tomove!=NULL))
   {
-    if (((QRING_CMD!=IDTYP(tomove)) && RingDependend(IDTYP(tomove)))
+    if (RingDependend(IDTYP(tomove))
     || ((IDTYP(tomove)==LIST_CMD) && (lRingDependend(IDLIST(tomove)))))
     {
       /*move 'tomove' to ring id's*/
@@ -646,19 +652,22 @@ const char * piProcinfo(procinfov pi, const char *request)
 
 BOOLEAN piKill(procinfov pi)
 {
-  Voice *p=currentVoice;
-  while (p!=NULL)
-  {
-    if (p->pi==pi && pi->ref <= 1)
-    {
-      Warn("`%s` in use, can not be killed",pi->procname);
-      return TRUE;
-    }
-    p=p->next;
-  }
   (pi->ref)--;
-  if (pi->ref <= 0)
+  if (pi->ref == 0)
   {
+    if (pi->language==LANG_SINGULAR)
+    {
+      Voice *p=currentVoice;
+      while (p!=NULL)
+      {
+        if (p->pi==pi && pi->ref <= 1)
+        {
+          Warn("`%s` in use, can not be killed",pi->procname);
+          return TRUE;
+        }
+        p=p->next;
+      }
+    }
     if (pi->libname != NULL) // OB: ????
       omFree((ADDRESS)pi->libname);
     if (pi->procname != NULL) // OB: ????
@@ -673,7 +682,7 @@ BOOLEAN piKill(procinfov pi)
     {
     }
     memset((void *) pi, 0, sizeof(procinfo));
-    pi->language=LANG_NONE;
+    //pi->language=LANG_NONE;
     omFreeBin((ADDRESS)pi,  procinfo_bin);
   }
   return FALSE;
@@ -703,8 +712,6 @@ void proclevel::push(char *n)
 {
   //Print("push %s\n",n);
   proclevel *p=(proclevel*)omAlloc0(sizeof(proclevel));
-  p->cRing=currRing;
-  p->cRingHdl=currRingHdl;
   p->name=n;
   p->cPackHdl=currPackHdl;
   p->cPack=currPack;
@@ -768,11 +775,9 @@ BOOLEAN iiAlias(leftv p)
   idhdl pp=(idhdl)p->data;
   switch(pp->typ)
   {
-#ifdef SINGULAR_4_1
       case CRING_CMD:
         nKillChar((coeffs)pp);
         break;
-#endif
       case DEF_CMD:
       case INT_CMD:
         break;
