@@ -77,7 +77,7 @@ poly ssiReadPoly_R(const ssiInfo *D, const ring r);
 ideal ssiReadIdeal_R(const ssiInfo *d,const ring r);
 
 // the helper functions:
-void ssiSetCurrRing(const ring r)
+BOOLEAN ssiSetCurrRing(const ring r) /* returned: not accepted */
 {
   //  if (currRing!=NULL)
   //  Print("need to change the ring, currRing:%s, switch to: ssiRing%d\n",IDID(currRingHdl),nr);
@@ -87,13 +87,29 @@ void ssiSetCurrRing(const ring r)
   {
     char name[20];
     int nr=0;
-    do
-    { sprintf(name,"ssiRing%d",nr); nr++; }
-    while(IDROOT->get(name, 0)!=NULL);
-    idhdl h=enterid(omStrDup(name),0,RING_CMD,&IDROOT,FALSE);
-    IDRING(h)=r;
-    r->ref++;
+    idhdl h=NULL;
+    loop
+    {
+      sprintf(name,"ssiRing%d",nr); nr++;
+      h=IDROOT->get(name, 0);
+      if (h==NULL)
+      {
+        h=enterid(omStrDup(name),0,RING_CMD,&IDROOT,FALSE);
+        IDRING(h)=r;
+        r->ref++;
+        break;
+      }
+      else if ((IDTYP(h)==RING_CMD)
+      && (rEqual(r,IDRING(h),1)))
+        break;
+    }
     rSetHdl(h);
+    return FALSE;
+  }
+  else
+  {
+    rKill(r);
+    return TRUE;
   }
 }
 // the implementation of the functions:
@@ -220,12 +236,12 @@ void ssiWriteRing_R(ssiInfo *d,const ring r)
     }
     else
     {
-      fprintf(d->f_write,"0 "/*ideal with 0 entries */);
+      fputs("0 ",d->f_write/*ideal with 0 entries */);
     }
   }
   else /* dummy ring r==NULL*/
   {
-    fprintf(d->f_write,"0 0 0 0 "/*,r->ch,r->N, blocks, q-ideal*/);
+    fputs("0 0 0 0 "/*,r->ch,r->N, blocks, q-ideal*/,d->f_write);
   }
 }
 
@@ -251,7 +267,7 @@ void ssiWriteRing(ssiInfo *d,const ring r)
   }
   ssiWriteRing_R(d,r);
 }
-void ssiWritePoly_R(const ssiInfo *d, int typ, poly p, const ring r)
+void ssiWritePoly_R(const ssiInfo *d, int /*typ*/, poly p, const ring r)
 {
   fprintf(d->f_write,"%d ",pLength(p));//number of terms
 
@@ -332,7 +348,7 @@ void ssiWriteProc(const ssiInfo *d,procinfov p)
 void ssiWriteList(si_link l,lists dd)
 {
   ssiInfo *d=(ssiInfo*)l->data;
-  int Ll=lSize(dd);
+  int Ll=dd->nr;
   fprintf(d->f_write,"%d ",Ll+1);
   int i;
   for(i=0;i<=Ll;i++)
@@ -375,9 +391,9 @@ char *ssiReadString(const ssiInfo *d)
   int l;
   l=s_readint(d->f_read);
   buf=(char*)omAlloc0(l+1);
-  int c =s_getc(d->f_read); /* skip ' '*/
-  int ll=s_readbytes(buf,l,d->f_read);
-  //if (ll!=l) printf("want %d, got %d bytes\n",l,ll);
+  int throwaway =s_getc(d->f_read); /* skip ' '*/
+  throwaway=s_readbytes(buf,l,d->f_read);
+  //if (throwaway!=l) printf("want %d, got %d bytes\n",l,throwaway);
   buf[l]='\0';
   return buf;
 }
@@ -423,6 +439,7 @@ number ssiReadBigInt(const ssiInfo *d)
 
 number ssiReadNumber(const ssiInfo *d)
 {
+  if (currRing==NULL) ssiSetCurrRing(d->r);
   return ssiReadNumber_CF(d,d->r->cf);
 }
 
@@ -439,7 +456,11 @@ ring ssiReadRing(const ssiInfo *d)
     char *cf_name=ssiReadString(d);
     cf=nFindCoeffByName(cf_name);
     if (cf==NULL)
-    { Werror("cannot find cf:%s",cf_name);return NULL;}
+    {
+      Werror("cannot find cf:%s",cf_name);
+      omFree(cf_name);
+      return NULL;
+    }
   }
   if (N!=0)
   {
@@ -452,13 +473,13 @@ ring ssiReadRing(const ssiInfo *d)
   // read the orderings:
   int num_ord; // number of orderings
   num_ord=s_readint(d->f_read);
-  int *ord=(int *)omAlloc0((num_ord+1)*sizeof(int));
+  rRingOrder_t *ord=(rRingOrder_t *)omAlloc0((num_ord+1)*sizeof(rRingOrder_t));
   int *block0=(int *)omAlloc0((num_ord+1)*sizeof(int));
   int *block1=(int *)omAlloc0((num_ord+1)*sizeof(int));
   int **wvhdl=(int**)omAlloc0((num_ord+1)*sizeof(int*));
   for(i=0;i<num_ord;i++)
   {
-    ord[i]=s_readint(d->f_read);
+    ord[i]=(rRingOrder_t)s_readint(d->f_read);
     block0[i]=s_readint(d->f_read);
     block1[i]=s_readint(d->f_read);
     switch(ord[i])
@@ -523,11 +544,21 @@ ring ssiReadRing(const ssiInfo *d)
     else
     {
       Werror("ssi: read unknown coeffs type (%d)",ch);
+      for(i=0;i<N;i++)
+      {
+        omFree(names[i]);
+      }
+      omFreeSize(names,N*sizeof(char*));
       return NULL;
     }
     ideal q=ssiReadIdeal_R(d,r);
     if (IDELEMS(q)==0) omFreeBin(q,sip_sideal_bin);
     else r->qideal=q;
+    for(i=0;i<N;i++)
+    {
+      omFree(names[i]);
+    }
+    omFreeSize(names,N*sizeof(char*));
     return r;
   }
 }
@@ -565,12 +596,13 @@ poly ssiReadPoly_R(const ssiInfo *D, const ring r)
 
 poly ssiReadPoly(const ssiInfo *D)
 {
-// < # of terms> < term1> < .....
+  if (currRing==NULL) ssiSetCurrRing(D->r);
   return ssiReadPoly_R(D,D->r);
 }
 
 ideal ssiReadIdeal_R(const ssiInfo *d,const ring r)
 {
+// < # of terms> < term1> < .....
   int n,i;
   ideal I;
   n=s_readint(d->f_read);
@@ -584,6 +616,7 @@ ideal ssiReadIdeal_R(const ssiInfo *d,const ring r)
 
 ideal ssiReadIdeal(const ssiInfo *d)
 {
+  if (currRing==NULL) ssiSetCurrRing(d->r);
   return ssiReadIdeal_R(d,d->r);
 }
 
@@ -664,12 +697,12 @@ lists ssiReadList(si_link l)
   ssiInfo *d=(ssiInfo*)l->data;
   int nr;
   nr=s_readint(d->f_read);
-  lists L=(lists)omAlloc(sizeof(*L));
+  lists L=(lists)omAlloc0Bin(slists_bin);
   L->Init(nr);
 
   int i;
   leftv v;
-  for(i=0;i<nr;i++)
+  for(i=0;i<=L->nr;i++)
   {
     v=ssiRead1(l);
     memcpy(&(L->m[i]),v,sizeof(*v));
@@ -716,8 +749,7 @@ bigintmat* ssiReadBigintmat(const ssiInfo *d)
 void ssiReadBlackbox(leftv res, si_link l)
 {
   ssiInfo *d=(ssiInfo*)l->data;
-  int throwaway;
-  throwaway=s_readint(d->f_read);
+  int throwaway=s_readint(d->f_read);
   char *name=ssiReadString(d);
   int tok;
   blackboxIsCmd(name,tok);
@@ -731,6 +763,7 @@ void ssiReadBlackbox(leftv res, si_link l)
   {
     Werror("blackbox %s not found",name);
   }
+  omFree(name);
 }
 
 void ssiReadAttrib(leftv res, si_link l)
@@ -747,7 +780,7 @@ void ssiReadAttrib(leftv res, si_link l)
   leftv tmp=ssiRead1(l);
   memcpy(res,tmp,sizeof(sleftv));
   memset(tmp,0,sizeof(sleftv));
-  omFreeSize(tmp,sizeof(sleftv));
+  omFreeBin(tmp,sleftv_bin);
   if (nr_of_attr>0)
   {
   }
@@ -777,6 +810,7 @@ BOOLEAN ssiOpen(si_link l, short flag, leftv u)
 
 
     SI_LINK_SET_OPEN_P(l, flag);
+    if(l->data!=NULL) omFreeSize(l->data,sizeof(ssiInfo));
     l->data=d;
     omFree(l->mode);
     l->mode = omStrDup(mode);
@@ -820,7 +854,6 @@ BOOLEAN ssiOpen(si_link l, short flag, leftv u)
             SI_LINK_SET_CLOSE_P(hh->l);
             ssiInfo *dd=(ssiInfo*)hh->l->data;
             s_close(dd->f_read);
-            s_free(dd->f_read);
             fclose(dd->f_write);
             if (dd->r!=NULL) rKill(dd->r);
             omFreeSize((ADDRESS)dd,(sizeof *dd));
@@ -1199,7 +1232,7 @@ BOOLEAN ssiClose(si_link l)
           }
         }
       }
-      if (d->f_read!=NULL) { s_close(d->f_read);s_free(d->f_read);d->f_read=NULL;}
+      if (d->f_read!=NULL) { s_close(d->f_read);d->f_read=NULL;}
       if (d->f_write!=NULL) { fclose(d->f_write); d->f_write=NULL; }
       if ((strcmp(l->mode,"tcp")==0)
       || (strcmp(l->mode,"fork")==0))
@@ -1237,7 +1270,7 @@ BOOLEAN ssiClose(si_link l)
 leftv ssiRead1(si_link l)
 {
   ssiInfo *d = (ssiInfo *)l->data;
-  leftv res=(leftv)omAlloc0(sizeof(sleftv));
+  leftv res=(leftv)omAlloc0Bin(sleftv_bin);
   int t=0;
   t=s_readint(d->f_read);
   //Print("got type %d\n",t);
@@ -1260,14 +1293,14 @@ leftv ssiRead1(si_link l)
              d->r=ssiReadRing(d);
              if (d->r==NULL) return NULL;
              res->data=(char*)d->r;
+             d->r->ref++;
              res->rtyp=RING_CMD;
-             // we are in the top-level, so set the basering to d->r:
-             if (d->r!=NULL)
+             if (t==15) // setring
              {
-               d->r->ref++;
-               ssiSetCurrRing(d->r);
+               if(ssiSetCurrRing(d->r)) { d->r=currRing; d->r->ref++; }
+               omFreeBin(res,sleftv_bin);
+               return ssiRead1(l);
              }
-             if (t==15) return ssiRead1(l);
            }
            break;
     case 6:res->rtyp=POLY_CMD;
@@ -1343,7 +1376,7 @@ leftv ssiRead1(si_link l)
                 n98_o2=s_readint(d->f_read);
                 if ((n98_v!=SSI_VERSION) ||(n98_m!=MAX_TOK))
                 {
-                  Print("incompatible versions of ssi: %d/%d vs %d/%d",
+                  Print("incompatible versions of ssi: %d/%d vs %d/%d\n",
                                   SSI_VERSION,MAX_TOK,n98_v,n98_m);
                 }
                 #ifndef SING_NDEBUG
@@ -1352,9 +1385,10 @@ leftv ssiRead1(si_link l)
                 #endif
                 si_opt_1=n98_o1;
                 si_opt_2=n98_o2;
+                omFreeBin(res,sleftv_bin);
                 return ssiRead1(l);
              }
-    case 99: ssiClose(l); m2_end(0);
+    case 99: omFreeBin(res,sleftv_bin); ssiClose(l); m2_end(0);
     case 0: if (s_iseof(d->f_read))
             {
               ssiClose(l);
@@ -1362,7 +1396,7 @@ leftv ssiRead1(si_link l)
             res->rtyp=DEF_CMD;
             break;
     default: Werror("not implemented (t:%d)",t);
-             omFreeSize(res,sizeof(sleftv));
+             omFreeBin(res,sleftv_bin);
              res=NULL;
              break;
   }
@@ -1372,11 +1406,11 @@ leftv ssiRead1(si_link l)
   && (currRing!=d->r)
   && (res->RingDependend()))
   {
-    ssiSetCurrRing(d->r);
+    if(ssiSetCurrRing(d->r)) { d->r=currRing; d->r->ref++; }
   }
   return res;
 no_ring: WerrorS("no ring");
-  omFreeSize(res,sizeof(sleftv));
+  omFreeBin(res,sleftv_bin);
   return NULL;
 }
 //**************************************************************************/
@@ -1785,6 +1819,7 @@ int ssiBatch(const char *host, const char * port)
   char *buf=(char*)omAlloc(256);
   sprintf(buf,"ssi:connect %s:%s",host,port);
   slInit(l, buf);
+  omFreeSize(buf,256);
   if (slOpen(l,SI_LINK_OPEN,NULL)) return 1;
   SI_LINK_SET_RW_OPEN_P(l);
 
