@@ -167,6 +167,8 @@ void syCompactify1(SSet sPairs, int* sPlength, int first)
 * exp[0]
 */
 
+#if 0
+// unused
 #ifdef PDEBUG
 static int syzcomp2dpc_test(poly p1, poly p2)
 {
@@ -227,6 +229,7 @@ static int syzcomp2dpc_test(poly p1, poly p2)
   return -1;
 }
 #endif // PDEBUG
+#endif
 
 poly syRedtail (poly p, syStrategy syzstr, int index)
 {
@@ -1295,6 +1298,8 @@ SSet syChosePairs(syStrategy syzstr, int *index, int *howmuch, int * actdeg)
   return syChosePairsPutIn(syzstr,index,howmuch,actdeg,0,syzstr->length);
 }
 
+#if 0
+// unused
 /*3
 * FOR THE INHOMOGENEOUS CASE ONLY!
 * looks through the pair set and the given module for
@@ -1316,6 +1321,7 @@ static SSet syChosePairsIH(syStrategy syzstr, int *index,
   }
   return result;
 }
+#endif
 
 /*3
 * looks through the pair set and the given module for
@@ -1509,13 +1515,8 @@ void syKillComputation(syStrategy syzstr, ring r)
       {
         if (syzstr->minres[i]!=NULL)
         {
-          for (j=0;j<IDELEMS(syzstr->minres[i]);j++)
-          {
-            if (syzstr->minres[i]->m[j]!=NULL)
-              p_Delete(&(syzstr->minres[i]->m[j]),r);
-          }
+          id_Delete(&(syzstr->minres[i]),r);
         }
-        id_Delete(&(syzstr->minres[i]),r);
       }
       omFreeSize((ADDRESS)syzstr->minres,(syzstr->length+1)*sizeof(ideal));
     }
@@ -1525,13 +1526,8 @@ void syKillComputation(syStrategy syzstr, ring r)
       {
         if (syzstr->fullres[i]!=NULL)
         {
-          for (j=0;j<IDELEMS(syzstr->fullres[i]);j++)
-          {
-            if (syzstr->fullres[i]->m[j]!=NULL)
-              p_Delete(&(syzstr->fullres[i]->m[j]),r);
-          }
+          id_Delete(&(syzstr->fullres[i]),r);
         }
-        id_Delete(&(syzstr->fullres[i]),r);
       }
       omFreeSize((ADDRESS)syzstr->fullres,(syzstr->length+1)*sizeof(ideal));
     }
@@ -1567,8 +1563,8 @@ void syKillComputation(syStrategy syzstr, ring r)
           {
             syzstr->orderedRes[i]->m[j] = NULL;
           }
+          id_Delete(&(syzstr->orderedRes[i]),sr);
         }
-        id_Delete(&(syzstr->orderedRes[i]),sr);
         if (syzstr->truecomponents[i]!=NULL)
         {
           omFreeSize((ADDRESS)syzstr->truecomponents[i],(IDELEMS(syzstr->res[i])+1)*sizeof(int));
@@ -1826,7 +1822,8 @@ intvec * syBettiOfComputation(syStrategy syzstr, BOOLEAN minim,int * row_shift,
 
   if ((result!=NULL)
   && ((minim) || (syzstr->resPairs!=NULL))
-  && std_weights)
+  && std_weights
+  && (syzstr->betti==NULL))
   {
     syzstr->betti = ivCopy(result); // cache the result...
   }
@@ -2036,13 +2033,14 @@ void syPrint(syStrategy syzstr, const char *sn)
   PrintLn();
   if (syzstr->minres==NULL)
   {
-    PrintS("resolution not minimized yet");
-    PrintLn();
+    PrintS("resolution not minimized yet\n");
   }
 
   if (syzstr->resolution == NULL) syzstr->resolution = resolution;
 }
 
+#if 0
+// unused
 /*2
 * deleting all monomials the component of which correspond
 * to non-minimal generators
@@ -2067,6 +2065,7 @@ static poly syStripOut(poly p,intvec * toStrip)
   }
   return p;
 }
+#endif
 
 /*2
 * copies only those monomials the component of which correspond
