@@ -28,6 +28,17 @@
 // #include<gmp.h>
 // #include<vector>
 
+# define omsai 1
+#if omsai
+
+#include<libpolys/polys/ext_fields/transext.h>
+#include<libpolys/coeffs/coeffs.h>
+#include<kernel/linear_algebra/linearAlgebra.h>
+#include <coeffs/numbers.h>
+#include <vector>
+#include <Singular/ipshell.h>
+
+#endif
 
 static int  **Qpol;
 static int  *Q0, *Ql;
@@ -68,7 +79,7 @@ static void hHilbEst(scfmon stc, int Nstc, varset var, int Nvar)
     {
       if (z>(MAX_INT_VAL)/2)
       {
-       WerrorS("interal arrays too big");
+       WerrorS("internal arrays too big");
        return;
       }
       p = (int *)omAlloc((unsigned long)z * sizeof(int));
@@ -228,22 +239,7 @@ static void hWDegree(intvec *wdegree)
 // ---------------------------------- ADICHANGES ---------------------------------------------
 //!!!!!!!!!!!!!!!!!!!!! Just for Monomial Ideals !!!!!!!!!!!!!!!!!!!!!!!!!!!!
 
-//returns the degree of the monomial
-static int DegMon(poly p)
-{
-    #if 1
-    int i,deg;
-    deg = 0;
-    for(i=1;i<=currRing->N;i++)
-    {
-        deg = deg + p_GetExp(p, i, currRing);
-    }
-    return(deg);
-    #else
-    return(p_Deg(p, currRing));
-    #endif
-}
-
+#if 0 // unused
 //Tests if the ideal is sorted by degree
 static bool idDegSortTest(ideal I)
 {
@@ -253,7 +249,7 @@ static bool idDegSortTest(ideal I)
     }
     for(int i = 0; i<IDELEMS(I)-1; i++)
     {
-        if(DegMon(I->m[i])>DegMon(I->m[i+1]))
+        if(p_Totaldegree(I->m[i],currRing)>p_Totaldegree(I->m[i+1],currRing))
         {
             idPrint(I);
             WerrorS("Ideal is not deg sorted!!");
@@ -262,6 +258,7 @@ static bool idDegSortTest(ideal I)
     }
     return(TRUE);
 }
+#endif
 
 //adds the new polynomial at the coresponding position
 //and simplifies the ideal
@@ -276,14 +273,14 @@ static ideal SortByDeg_p(ideal I, poly p)
     }
     idSkipZeroes(I);
     #if 1
-    for(i = 0; (i<IDELEMS(I)) && (DegMon(I->m[i])<=DegMon(p)); i++)
+    for(i = 0; (i<IDELEMS(I)) && (p_Totaldegree(I->m[i],currRing)<=p_Totaldegree(p,currRing)); i++)
     {
         if(p_DivisibleBy( I->m[i],p, currRing))
         {
             return(I);
         }
     }
-    for(i = IDELEMS(I)-1; (i>=0) && (DegMon(I->m[i])>=DegMon(p)); i--)
+    for(i = IDELEMS(I)-1; (i>=0) && (p_Totaldegree(I->m[i],currRing)>=p_Totaldegree(p,currRing)); i--)
     {
         if(p_DivisibleBy(p,I->m[i], currRing))
         {
@@ -299,9 +296,9 @@ static ideal SortByDeg_p(ideal I, poly p)
     #endif
     idSkipZeroes(I);
     //First I take the case when all generators have the same degree
-    if(DegMon(I->m[0]) == DegMon(I->m[IDELEMS(I)-1]))
+    if(p_Totaldegree(I->m[0],currRing) == p_Totaldegree(I->m[IDELEMS(I)-1],currRing))
     {
-        if(DegMon(p)<DegMon(I->m[0]))
+        if(p_Totaldegree(p,currRing)<p_Totaldegree(I->m[0],currRing))
         {
             idInsertPoly(I,p);
             idSkipZeroes(I);
@@ -312,14 +309,14 @@ static ideal SortByDeg_p(ideal I, poly p)
             I->m[0] = p;
             return(I);
         }
-        if(DegMon(p)>=DegMon(I->m[IDELEMS(I)-1]))
+        if(p_Totaldegree(p,currRing)>=p_Totaldegree(I->m[IDELEMS(I)-1],currRing))
         {
             idInsertPoly(I,p);
             idSkipZeroes(I);
             return(I);
         }
     }
-    if(DegMon(p)<=DegMon(I->m[0]))
+    if(p_Totaldegree(p,currRing)<=p_Totaldegree(I->m[0],currRing))
     {
         idInsertPoly(I,p);
         idSkipZeroes(I);
@@ -330,7 +327,7 @@ static ideal SortByDeg_p(ideal I, poly p)
         I->m[0] = p;
         return(I);
     }
-    if(DegMon(p)>=DegMon(I->m[IDELEMS(I)-1]))
+    if(p_Totaldegree(p,currRing)>=p_Totaldegree(I->m[IDELEMS(I)-1],currRing))
     {
         idInsertPoly(I,p);
         idSkipZeroes(I);
@@ -338,7 +335,7 @@ static ideal SortByDeg_p(ideal I, poly p)
     }
     for(i = IDELEMS(I)-2; ;)
     {
-        if(DegMon(p)==DegMon(I->m[i]))
+        if(p_Totaldegree(p,currRing)==p_Totaldegree(I->m[i],currRing))
         {
             idInsertPoly(I,p);
             idSkipZeroes(I);
@@ -349,7 +346,7 @@ static ideal SortByDeg_p(ideal I, poly p)
             I->m[i] = p;
             return(I);
         }
-        if(DegMon(p)>DegMon(I->m[i]))
+        if(p_Totaldegree(p,currRing)>p_Totaldegree(I->m[i],currRing))
         {
             idInsertPoly(I,p);
             idSkipZeroes(I);
@@ -423,7 +420,7 @@ ideal idQuotMon(ideal Iorig, ideal p)
             }
         }
         p_Setm(res->m[i], currRing);
-        if(DegMon(res->m[i]) == DegMon(I->m[i]))
+        if(p_Totaldegree(res->m[i],currRing) == p_Totaldegree(I->m[i],currRing))
         {
             res->m[i] = NULL; // pDelete
         }
@@ -489,6 +486,7 @@ static poly ChoosePVar (ideal I)
     return(NULL); //i.e. it is the maximal ideal
 }
 
+#if 0 // unused
 //choice XL: last entry divided by x (xy10z15 -> y9z14)
 static poly ChoosePXL(ideal I)
 {
@@ -521,7 +519,9 @@ static poly ChoosePXL(ideal I)
     m = ChoosePVar(I);
     return(m);
 }
+#endif
 
+#if 0 // unused
 //choice XF: first entry divided by x (xy10z15 -> y9z14)
 static poly ChoosePXF(ideal I)
 {
@@ -554,7 +554,9 @@ static poly ChoosePXF(ideal I)
     m = ChoosePVar(I);
     return(m);
 }
+#endif
 
+#if 0 // unused
 //choice OL: last entry the first power (xy10z15 -> xy9z15)
 static poly ChoosePOL(ideal I)
 {
@@ -584,7 +586,9 @@ static poly ChoosePOL(ideal I)
     m = ChoosePVar(I);
     return(m);
 }
+#endif
 
+#if 0 // unused
 //choice OF: first entry the first power (xy10z15 -> xy9z15)
 static poly ChoosePOF(ideal I)
 {
@@ -614,7 +618,9 @@ static poly ChoosePOF(ideal I)
     m = ChoosePVar(I);
     return(m);
 }
+#endif
 
+#if 0 // unused
 //choice VL: last entry the first variable with power (xy10z15 -> y)
 static poly ChoosePVL(ideal I)
 {
@@ -642,7 +648,9 @@ static poly ChoosePVL(ideal I)
     m = ChoosePVar(I);
     return(m);
 }
+#endif
 
+#if 0 // unused
 //choice VF: first entry the first variable with power (xy10z15 -> y)
 static poly ChoosePVF(ideal I)
 {
@@ -670,6 +678,7 @@ static poly ChoosePVF(ideal I)
     m = ChoosePVar(I);
     return(m);
 }
+#endif
 
 //choice JL: last entry just variable with power (xy10z15 -> y10)
 static poly ChoosePJL(ideal I)
@@ -699,6 +708,7 @@ static poly ChoosePJL(ideal I)
     return(m);
 }
 
+#if 0
 //choice JF: last entry just variable with power -1 (xy10z15 -> y9)
 static poly ChoosePJF(ideal I)
 {
@@ -726,6 +736,7 @@ static poly ChoosePJF(ideal I)
     m = ChoosePVar(I);
     return(m);
 }
+#endif
 
 //chooses 1 \neq p \not\in S. This choice should be made optimal
 static poly ChooseP(ideal I)
@@ -748,7 +759,7 @@ static poly SearchP(ideal I)
 {
     int i,j,exp;
     poly res;
-    if(DegMon(I->m[IDELEMS(I)-1])<=1)
+    if(p_Totaldegree(I->m[IDELEMS(I)-1],currRing)<=1)
     {
         res = ChoosePVar(I);
         return(res);
@@ -792,7 +803,7 @@ static bool JustVar(ideal I)
     }
     return(TRUE);
     #else
-    if(DegMon(I->m[IDELEMS(I)-1])>1)
+    if(p_Totaldegree(I->m[IDELEMS(I)-1],currRing)>1)
     {
         return(FALSE);
     }
@@ -848,7 +859,7 @@ static poly SqFree (ideal I)
     int i,j;
     bool flag=TRUE;
     poly notsqrfree = NULL;
-    if(DegMon(I->m[IDELEMS(I)-1])<=1)
+    if(p_Totaldegree(I->m[IDELEMS(I)-1],currRing)<=1)
     {
         return(notsqrfree);
     }
@@ -945,7 +956,7 @@ void rouneslice(ideal I, ideal S, poly q, poly x, int &prune, int &moreprune, in
     int i,j;
     int dummy;
     poly m;
-    ideal p, koszsimp;
+    ideal p;
     //----------- PRUNING OF S ---------------
     //S SHOULD IN THIS POINT BE ORDERED BY DEGREE
     for(i=IDELEMS(S)-1;i>=0;i--)
@@ -1042,15 +1053,15 @@ void rouneslice(ideal I, ideal S, poly q, poly x, int &prune, int &moreprune, in
                 mpz_init( &hilbertcoef[NNN]);
                 mpz_set(  &hilbertcoef[NNN], ec);
                 mpz_clear(ec);
-                hilbpower[NNN] = DegMon(q);
+                hilbpower[NNN] = p_Totaldegree(q,currRing);
                 NNN++;
             }
         else
         {
             //I look if the power appears already
-            for(i = 0;(i<NNN)&&(flag == FALSE)&&(DegMon(q)>=hilbpower[i]);i++)
+            for(i = 0;(i<NNN)&&(flag == FALSE)&&(p_Totaldegree(q,currRing)>=hilbpower[i]);i++)
             {
-                if((hilbpower[i]) == (DegMon(q)))
+                if((hilbpower[i]) == (p_Totaldegree(q,currRing)))
                 {
                     flag = TRUE;
                     mpz_add(&hilbertcoef[i],&hilbertcoef[i],ec_ptr);
@@ -1068,7 +1079,7 @@ void rouneslice(ideal I, ideal S, poly q, poly x, int &prune, int &moreprune, in
                 }
                 mpz_set(  &hilbertcoef[i], ec);
                 mpz_clear(ec);
-                hilbpower[i] = DegMon(q);
+                hilbpower[i] = p_Totaldegree(q,currRing);
                 NNN++;
             }
         }
@@ -1400,5 +1411,754 @@ void hLookSeries(ideal S, intvec *modulweight, ideal Q, intvec *wdegree, ring ta
   delete hseries2;
 }
 
+/***********************************************************************
+ Computation of Hilbert series of non-commutative monomial algebras
+************************************************************************/
 
 
+static void idInsertMonomials(ideal I, poly p)
+{
+  /*
+   * adds monomial in I and if required,
+   * enlarges the size of poly-set by 16
+   * does not make copy of  p
+   */
+
+  if(I == NULL)
+  {
+    return;
+  }
+
+  int j = IDELEMS(I) - 1;
+  while ((j >= 0) && (I->m[j] == NULL))
+  {
+    j--;
+  }
+  j++;
+  if (j == IDELEMS(I))
+  {
+    pEnlargeSet(&(I->m), IDELEMS(I), 16);
+    IDELEMS(I) +=16;
+  }
+  I->m[j] = p;
+}
+
+static int isMonoIdBasesSame(ideal J, ideal Ob)
+{
+  /*
+   * polynomials of J and Ob are assumed to
+   * be already sorted. J and Ob are
+   * represented by the minimal generating set
+   */
+  int i, s;
+  s = 1;
+  int JCount = IDELEMS(J);
+  int ObCount = IDELEMS(Ob);
+
+  if(idIs0(J))
+  {
+    return(1);
+  }
+  if(JCount != ObCount)
+  {
+    return(0);
+  }
+
+  for(i = 0; i < JCount; i++)
+  {
+    if(!(p_LmEqual(J->m[i], Ob->m[i], currRing)))
+    {
+      return(0);
+    }
+  }
+  return(s);
+}
+
+static int CountOnIdUptoTruncationIndex(ideal I, int tr)
+{
+  /*
+   * I must be sorted in ascending order
+   * counts the number of polys in I upto
+   * degree less or equal to tr
+   */
+
+  //case when I=1;
+  if(p_Totaldegree(I->m[0], currRing) == 0)
+  {
+    return(1);
+  }
+
+  int count = 0;
+  for(int i = 0; i < IDELEMS(I); i++)
+  {
+    if(p_Totaldegree(I->m[i], currRing) > tr)
+    {
+      return (count);
+    }
+    count = count + 1;
+  }
+
+  return(count);
+}
+
+static int isMonoIdBasesSame_IG_Case(ideal J, int JCount, ideal Ob, int ObCount)
+{
+  /*
+   * polynomials of J and obc are assumed to
+   * be already sorted. J and Ob are
+   * represented by the minimal generating set.
+   * checks if J and Ob are same in polys upto deg <=tr
+   */
+
+  int i, s;
+  s = 1;
+  //when J is null
+  if(JCount == 0)
+  {
+    return(1);
+  }
+
+  if(JCount != ObCount)
+  {
+    return(0);
+  }
+
+  for(i = 0; i< JCount; i++)
+  {
+    if(!(p_LmEqual(J->m[i], Ob->m[i], currRing)))
+    {
+      return(0);
+    }
+  }
+
+  return(s);
+}
+
+static int positionInOrbit_IG_Case(ideal I, poly w, std::vector<ideal> idorb, std::vector<poly> polist, int trInd)
+{
+  /*
+   * compares the ideal I with ideals in the Orbit 'idorb'
+   * upto degree trInd - max(deg of w, deg of word in polist) polynomials;
+   * I and ideals in the Orbit are sorted,
+   * Orbit is ordered,
+   *
+   * returns 0 if I is not equal to any of the ideals
+   * in the Orbit else returns position of the matched ideal
+   */
+
+  int ps = 0;
+  int i, s = 0;
+  int orbCount = idorb.size();
+
+  if(idIs0(I))
+  {
+    return(1);
+  }
+
+  int degw = p_Totaldegree(w, currRing);
+  int degp;
+  int dtr;
+  int dtrp;
+
+  dtr = trInd - degw;
+  int IwCount;
+
+   IwCount = CountOnIdUptoTruncationIndex(I, dtr);
+
+  if(IwCount == 0)
+  {
+    return(1);
+  }
+
+  int ObCount;
+
+  bool flag2 = FALSE;
+
+  for(i = 1;i < orbCount; i++)
+  {
+    degp = p_Totaldegree(polist[i], currRing);
+    if(degw > degp)
+    {
+      dtr = trInd - degw;
+
+      ObCount = 0;
+      ObCount = CountOnIdUptoTruncationIndex(idorb[i], dtr);
+      if(ObCount == 0)
+      {continue;}
+      if(flag2)
+      {
+        IwCount = 0;
+        IwCount = CountOnIdUptoTruncationIndex(I, dtr);
+        flag2 = FALSE;
+      }
+    }
+    else
+    {
+      flag2 = TRUE;
+      dtrp = trInd - degp;
+      ObCount = 0;
+      ObCount = CountOnIdUptoTruncationIndex(idorb[i], dtrp);
+      IwCount = 0;
+      IwCount = CountOnIdUptoTruncationIndex(I, dtrp);
+    }
+
+    s = isMonoIdBasesSame_IG_Case(I, IwCount, idorb[i], ObCount);
+
+    if(s)
+    {
+      ps = i + 1;
+      break;
+    }
+  }
+  return(ps);
+}
+
+static int positionInOrbit_FG_Case(ideal I, poly, std::vector<ideal> idorb, std::vector<poly>, int)
+{
+  /*
+   * compares the ideal I with ideals in the Orbit 'idorb'
+   * I and ideals in the Orbit are sorted,
+   * Orbit is ordered,
+   *
+   * returns 0 if I is not equal to any of the ideals
+   * in the Orbit else returns position of the matched ideal
+   */
+  int ps = 0;
+  int i, s = 0;
+  int OrbCount = idorb.size();
+
+  if(idIs0(I))
+  {
+    return(1);
+  }
+
+  for(i = 1; i < OrbCount; i++)
+  {
+    s = isMonoIdBasesSame(I, idorb[i]);
+    if(s)
+    {
+      ps = i + 1;
+      break;
+    }
+  }
+
+  return(ps);
+}
+
+static int monCompare( const void *m, const void *n)
+{
+  /* compares monomials */
+
+ return(p_Compare(*(poly*) m, *(poly*)n, currRing));
+}
+
+void sortMonoIdeal_pCompare(ideal I)
+{
+  /*
+   * sorts the monomial ideal in ascending order
+   * order must be a total degree
+   */
+
+  qsort(I->m, IDELEMS(I), sizeof(poly), monCompare);
+
+}
+
+
+
+static ideal  minimalMonomialsGenSet(ideal I)
+{
+  /*
+   * eliminates monomials which
+   * can be generated by others in I
+   */
+  //first sort monomials of the ideal
+
+  idSkipZeroes(I);
+
+  sortMonoIdeal_pCompare(I);
+
+  int i, k;
+  int ICount = IDELEMS(I);
+
+  for(k = ICount - 1; k >=1; k--)
+  {
+    for(i = 0; i < k; i++)
+    {
+
+      if(p_LmDivisibleBy(I->m[i], I->m[k], currRing))
+      {
+        pDelete(&(I->m[k]));
+        break;
+      }
+    }
+  }
+
+  idSkipZeroes(I);
+  return(I);
+}
+
+static poly shiftInMon(poly p, int i, int lV, const ring r)
+{
+  /*
+   * shifts the varibles of monomial p in the  i^th layer,
+   * p remains unchanged,
+   * creates new poly and returns it for the colon ideal
+   */
+  poly smon = p_One(r);
+  int j, sh, cnt;
+  cnt = r->N;
+  sh = i*lV;
+  int *e=(int *)omAlloc((r->N+1)*sizeof(int));
+  int *s=(int *)omAlloc0((r->N+1)*sizeof(int));
+  p_GetExpV(p, e, r);
+
+  for(j = 1; j <= cnt; j++)
+  {
+    if(e[j] == 1)
+    {
+      s[j+sh] = e[j];
+    }
+  }
+
+  p_SetExpV(smon, s, currRing);
+  omFree(e);
+  omFree(s);
+
+  p_SetComp(smon, p_GetComp(p, currRing), currRing);
+  p_Setm(smon, currRing);
+
+  return(smon);
+}
+
+static poly deleteInMon(poly w, int i, int lV, const ring r)
+{
+  /*
+   * deletes the variables upto i^th layer of monomial w
+   * w remains unchanged
+   * creates new poly and returns it for the colon ideal
+   */
+
+  poly dw = p_One(currRing);
+  int *e = (int *)omAlloc((r->N+1)*sizeof(int));
+  int *s=(int *)omAlloc0((r->N+1)*sizeof(int));
+  p_GetExpV(w, e, r);
+  int j, cnt;
+  cnt = i*lV;
+  /*
+  for(j=1;j<=cnt;j++)
+  {
+    e[j]=0;
+  }*/
+  for(j = (cnt+1); j < (r->N+1); j++)
+  {
+    s[j] = e[j];
+  }
+
+  p_SetExpV(dw, s, currRing);//new exponents
+  omFree(e);
+  omFree(s);
+
+  p_SetComp(dw, p_GetComp(w, currRing), currRing);
+  p_Setm(dw, currRing);
+
+  return(dw);
+}
+
+static void TwordMap(poly p, poly w, int lV, int d, ideal Jwi, bool &flag)
+{
+  /*
+   * computes T_w(p) in a new poly object and places it
+   * in a colon ideal Jwi of I
+   * p and w remain unchanged
+   * the new polys for Jwi are constructed by sub-routines
+   * deleteInMon, shiftInMon, p_Divide,
+   * places the result in Jwi and deletes the new polys
+   * coming in dw, smon, qmon
+   */
+  int i;
+  poly smon, dw;
+  poly qmonp = NULL;
+  bool del;
+
+  for(i = 0;i <= d - 1; i++)
+  {
+    dw = deleteInMon(w, i, lV, currRing);
+    smon = shiftInMon(p, i, lV, currRing);
+    del = TRUE;
+
+    if(pLmDivisibleBy(smon, w))
+    {
+      flag = TRUE;
+      del  = FALSE;
+
+      pDelete(&dw);
+      pDelete(&smon);
+
+      //delete all monomials of Jwi
+      //and make Jwi =1
+
+      for(int j = 0;j < IDELEMS(Jwi); j++)
+      {
+        pDelete(&Jwi->m[j]);
+      }
+
+      idInsertMonomials(Jwi, p_One(currRing));
+      break;
+    }
+
+    if(pLmDivisibleBy(dw, smon))
+    {
+      del = FALSE;
+      qmonp = p_Divide(smon, dw, currRing);
+      idInsertMonomials(Jwi, shiftInMon(qmonp, -d, lV, currRing));
+
+      //shiftInMon(qmonp, -d, lV, currRing):returns a new poly,
+      //qmonp remains unchanged, delete it
+      pDelete(&qmonp);
+      pDelete(&dw);
+      pDelete(&smon);
+    }
+    //in case both if are false, delete dw and smon
+    if(del)
+    {
+      pDelete(&dw);
+      pDelete(&smon);
+    }
+  }
+
+}
+
+static ideal colonIdeal(ideal S, poly w, int lV, ideal Jwi)
+{
+  /*
+   * computes the colon ideal of two-sided ideal S
+   * w.r.t. word w and save it on Jwi
+   * keeps S and w unchanged
+   */
+
+  if(idIs0(S))
+  {
+    return(S);
+  }
+
+  int i, d;
+  d = p_Totaldegree(w, currRing);
+  bool flag = FALSE;
+  int SCount = IDELEMS(S);
+  for(i = 0; i < SCount; i++)
+  {
+    TwordMap(S->m[i], w, lV, d, Jwi, flag);
+    if(flag)
+    {
+      break;
+    }
+  }
+
+  Jwi = minimalMonomialsGenSet(Jwi);
+  return(Jwi);
+}
+
+void HilbertSeries_OrbitData(ideal S, int lV, bool IG_CASE, bool mgrad, bool odp)
+{
+  /*
+   * It is based on iterative right colon operation to the
+   * monomial ideals of the free associative algebras.
+   * The algorithm terminates for the monomial right
+   * ideals whose monomials define regular formal language,
+   * that is, all the monomials of ideal can be obtained from
+   * finite subsets by applying the finite number
+   * of elementary operations.
+   */
+
+  int trInd;
+  S = minimalMonomialsGenSet(S);
+
+  int (*POS)(ideal, poly, std::vector<ideal>, std::vector<poly>, int);
+  if(IG_CASE)
+  {
+    if(idIs0(S))
+    {
+      WerrorS("wrong input:not the infinitely gen. case");
+        return;
+    }
+    trInd = p_Totaldegree(S->m[IDELEMS(S)-1], currRing);
+    POS = &positionInOrbit_IG_Case;
+  }
+  else
+  {
+    POS = &positionInOrbit_FG_Case;
+  }
+
+  std::vector<ideal > idorb;
+  std::vector< poly > polist;
+
+  ideal orb_init = idInit(1, 1);
+  idorb.push_back(orb_init);
+
+  polist.push_back( p_One(currRing));
+
+  std::vector< std::vector<int> > posMat;
+  std::vector<int> posRow(lV,0);
+  std::vector<int> C;
+
+  int ds, is, ps;
+  int lpcnt = 0;
+
+  poly w, wi;
+  ideal Jwi;
+
+  while(lpcnt < idorb.size())
+  {
+    w = NULL;
+    w = polist[lpcnt];
+
+    if(lpcnt >= 1)
+    {
+      if(p_Totaldegree(idorb[lpcnt]->m[0], currRing) != 0)
+      {
+        C.push_back(1);
+      }
+      else
+        C.push_back(0);
+    }
+    else
+      C.push_back(1);
+
+    ds = p_Totaldegree(w, currRing);
+    lpcnt++;
+
+    for(is = 1; is <= lV; is++)
+    {
+      wi = NULL;
+      //make new copy of word w=polist[lpcnt];
+      //in wi and update it (next colon word)
+      //if corresponding to wi get a new ideal(colon of S),
+      //keep it in the polist else delete it
+
+      wi = pCopy(w);
+      p_SetExp(wi, (ds*lV)+is, 1, currRing);
+      p_Setm(wi, currRing);
+
+      Jwi = NULL;
+      //Jwi stores colon ideal of S w.r.t. wi
+      //if get a new ideal place it in the idorb
+      //otherwise delete it
+      Jwi = idInit(1,1);
+
+      Jwi = colonIdeal(S, wi, lV, Jwi);
+      ps = (*POS)(Jwi, wi, idorb, polist, trInd);
+
+      if(ps == 0)  // finds a new ideal
+      {
+        posRow[is-1] = idorb.size();
+
+        idorb.push_back(Jwi);
+        polist.push_back(wi);
+      }
+      else // ideal is already there  in the orbit
+      {
+        posRow[is-1]=ps-1;
+        idDelete(&Jwi);
+        pDelete(&wi);
+      }
+    }
+    posMat.push_back(posRow);
+    posRow.resize(lV,0);
+  }
+  int lO = C.size();//size of the orbit
+  PrintLn();
+  Print("Maximal length of words = %ld\n", p_Totaldegree(polist[lO-1], currRing));
+  Print("\nOrbit length = %d\n", lO);
+  PrintLn();
+
+  if(odp)
+  {
+    Print("Words description of the Orbit: \n");
+    for(is = 0; is < lO; is++)
+    {
+      pWrite0(polist[is]);
+      PrintS("    ");
+    }
+    PrintLn();
+  }
+
+  for(is = idorb.size()-1; is >= 0; is--)
+  {
+    idDelete(&idorb[is]);
+  }
+  for(is = polist.size()-1; is >= 0; is--)
+  {
+    pDelete(&polist[is]);
+  }
+
+  idorb.resize(0);
+  polist.resize(0);
+
+  int adjMatrix[lO][lO];
+  memset(adjMatrix, 0, lO*lO*sizeof(int));
+  int rowCount, colCount;
+  int tm = 0;
+  if(!mgrad)
+  {
+    for(rowCount = 0; rowCount < lO; rowCount++)
+    {
+      for(colCount = 0; colCount < lV; colCount++)
+      {
+        tm = posMat[rowCount][colCount];
+        adjMatrix[rowCount][tm] = adjMatrix[rowCount][tm] + 1;
+      }
+    }
+  }
+
+  ring r = currRing;
+  int npar;
+  char** tt;
+  TransExtInfo p;
+  if(!mgrad)
+  {
+    tt=(char**)omalloc(sizeof(char*));
+    tt[0] = omStrDup("t");
+    npar = 1;
+  }
+  else
+  {
+    tt=(char**)omalloc(lV*sizeof(char*));
+    for(is = 0; is < lV; is++)
+    {
+      tt[is] = (char*)omalloc(7*sizeof(char)); //if required enlarge it later
+      sprintf (tt[is], "t(%d)", is+1);
+    }
+    npar = lV;
+  }
+
+  p.r = rDefault(0, npar, tt);
+  coeffs cf = nInitChar(n_transExt, &p);
+  char** xx = (char**)omalloc(sizeof(char*));
+  xx[0] = omStrDup("x");
+  ring R = rDefault(cf, 1, xx);
+  rChangeCurrRing(R);//rWrite(R);
+  /*
+   * matrix corresponding to the orbit of the ideal
+   */
+  matrix mR = mpNew(lO, lO);
+  matrix cMat = mpNew(lO,1);
+  poly rc;
+
+  if(!mgrad)
+  {
+    for(rowCount = 0; rowCount < lO; rowCount++)
+    {
+      for(colCount = 0; colCount < lO; colCount++)
+      {
+        if(adjMatrix[rowCount][colCount] != 0)
+        {
+          MATELEM(mR, rowCount + 1, colCount + 1) = p_ISet(adjMatrix[rowCount][colCount], R);
+          p_SetCoeff(MATELEM(mR, rowCount + 1, colCount + 1), n_Mult(pGetCoeff(mR->m[lO*rowCount+colCount]),n_Param(1, R->cf), R->cf), R);
+        }
+      }
+    }
+  }
+  else
+  {
+     for(rowCount = 0; rowCount < lO; rowCount++)
+     {
+       for(colCount = 0; colCount < lV; colCount++)
+       {
+          rc=NULL;
+          rc=p_One(R);
+          p_SetCoeff(rc, n_Mult(pGetCoeff(rc), n_Param(colCount+1, R->cf),R->cf), R);
+          MATELEM(mR, rowCount +1, posMat[rowCount][colCount]+1)=p_Add_q(rc,MATELEM(mR, rowCount +1, posMat[rowCount][colCount]+1), R);
+       }
+     }
+  }
+
+  for(rowCount = 0; rowCount < lO; rowCount++)
+  {
+    if(C[rowCount] != 0)
+    {
+      MATELEM(cMat, rowCount + 1, 1) = p_ISet(C[rowCount], R);
+    }
+  }
+
+  matrix u;
+  unitMatrix(lO, u); //unit matrix
+  matrix gMat = mp_Sub(u, mR, R);
+  char* s;
+  if(odp)
+  {
+    PrintS("\nlinear system:\n");
+    if(!mgrad)
+    {
+      for(rowCount = 0; rowCount < lO; rowCount++)
+      {
+        Print("H(%d) = ", rowCount+1);
+        for(colCount = 0; colCount < lV; colCount++)
+        {
+          StringSetS(""); nWrite(n_Param(1, R->cf));
+          s = StringEndS(); PrintS(s);
+          Print("*"); omFree(s);
+          Print("H(%d) + ", posMat[rowCount][colCount] + 1);
+        }
+        Print(" %d\n", C[rowCount] );
+      }
+      PrintS("where H(1) represents the series corresp. to input ideal\n");
+      PrintS("and i^th summand in the rhs of an eqn. is according\n");
+      PrintS("to the right colon map corresp. to the i^th variable\n");
+    }
+    else
+    {
+      for(rowCount = 0; rowCount < lO; rowCount++)
+      {
+        Print("H(%d) = ", rowCount+1);
+        for(colCount = 0; colCount < lV; colCount++)
+        {
+           StringSetS(""); nWrite(n_Param(colCount+1, R->cf));
+           s = StringEndS(); PrintS(s);
+           Print("*");omFree(s);
+           Print("H(%d) + ", posMat[rowCount][colCount] + 1);
+        }
+        Print(" %d\n", C[rowCount] );
+      }
+      PrintS("where H(1) represents the series corresp. to input ideal\n");
+    }
+  }
+  posMat.resize(0);
+  C.resize(0);
+  matrix pMat;
+  matrix lMat;
+  matrix uMat;
+  luDecomp(gMat, pMat, lMat, uMat, R);
+  matrix H_serVec = mpNew(lO, 1);
+  matrix Hnot;
+  luSolveViaLUDecomp(pMat, lMat, uMat, cMat, H_serVec, Hnot);
+
+  mp_Delete(&mR, R);
+  mp_Delete(&u, R);
+  mp_Delete(&pMat, R);
+  mp_Delete(&lMat, R);
+  mp_Delete(&uMat, R);
+  mp_Delete(&cMat, R);
+  mp_Delete(&gMat, R);
+  mp_Delete(&Hnot, R);
+  //print the Hilbert series and Orbit length
+  PrintLn();
+  Print("Hilbert series:");
+  PrintLn();
+  pWrite(H_serVec->m[0]);
+  PrintLn();
+  if(!mgrad)
+  {
+    omFree(tt[0]);
+  }
+  else
+  {
+    for(is = lV-1; is >= 0; is--)
+
+      omFree( tt[is]);
+  }
+  omFree(tt);
+  omFree(xx[0]);
+  omFree(xx);
+  rChangeCurrRing(r);
+  rKill(R);
+}
