@@ -71,10 +71,7 @@ void     nlPower(number x, int exp, number *lu, const coeffs r);
 const char *   nlRead (const char *s, number *a, const coeffs r);
 void     nlWrite(number a, const coeffs r);
 
-number   nlGetDenom(number &n, const coeffs r);
-number   nlGetNumerator(number &n, const coeffs r);
 void     nlCoeffWrite(const coeffs r, BOOLEAN details);
-number   nlChineseRemainder(number *x, number *q,int rl, const coeffs C);
 number   nlFarey(number nN, number nP, const coeffs CF);
 
 #ifdef LDEBUG
@@ -212,7 +209,6 @@ number nlMapGMP(number from, const coeffs /*src*/, const coeffs /*dst*/)
   z->debug=123456;
 #endif
   mpz_init_set(z->z,(mpz_ptr) from);
-  //mpz_init_set_ui(&z->n,1);
   z->s = 3;
   z=nlShort3(z);
   return z;
@@ -469,14 +465,16 @@ static number nlMapLongR(number from, const coeffs src, const coeffs dst)
 #endif
   dest = res->z;
 
+  void* (*allocfunc) (size_t);
+  mp_get_memory_functions (&allocfunc,NULL, NULL);
   if (e<0)
   {
     al = dest->_mp_size = size;
     if (al<2) al = 2;
-    dd = (mp_ptr)omAlloc(sizeof(mp_limb_t)*al);
+    dd = (mp_ptr)allocfunc(sizeof(mp_limb_t)*al);
     for (i=0;i<size;i++) dd[i] = qp[i];
     bl = 1-e;
-    nn = (mp_ptr)omAlloc0(sizeof(mp_limb_t)*bl);
+    nn = (mp_ptr)allocfunc(sizeof(mp_limb_t)*bl);
     nn[bl-1] = 1;
     ndest = res->n;
     ndest->_mp_d = nn;
@@ -487,7 +485,7 @@ static number nlMapLongR(number from, const coeffs src, const coeffs dst)
   {
     al = dest->_mp_size = size+e;
     if (al<2) al = 2;
-    dd = (mp_ptr)omAlloc(sizeof(mp_limb_t)*al);
+    dd = (mp_ptr)allocfunc(sizeof(mp_limb_t)*al);
     for (i=0;i<size;i++) dd[i+e] = qp[i];
     for (i=0;i<e;i++) dd[i] = 0;
     res->s = 3;
@@ -578,7 +576,7 @@ int nlSize(number a, const coeffs)
   if (a==INT_TO_SR(0))
      return 0; /* rational 0*/
   if (SR_HDL(a) & SR_INT)
-     return 1; /* immidiate int */
+     return 1; /* immediate int */
   int s=a->z[0]._mp_alloc;
 //  while ((s>0) &&(a->z._mp_d[s]==0L)) s--;
 //#if SIZEOF_LONG == 8
@@ -674,7 +672,7 @@ number nlInvers(number a, const coeffs r)
     n->s=1;
     if (((long)a)>0L)
     {
-      mpz_init_set_si(n->z,1L);
+      mpz_init_set_ui(n->z,1L);
       mpz_init_set_si(n->n,(long)SR_TO_INT(a));
     }
     else
@@ -719,7 +717,7 @@ number nlInvers(number a, const coeffs r)
               }
               else
               {
-                mpz_init_set_si(n->z,1L);
+                mpz_init_set_ui(n->z,1L);
               }
               break;
     }
@@ -753,7 +751,13 @@ number   nlExactDiv(number a, number b, const coeffs r)
     long bb=SR_TO_INT(b);
     return INT_TO_SR(aa/bb);
   }
+  number aa=NULL;
   number bb=NULL;
+  if (SR_HDL(a) & SR_INT)
+  {
+    aa=nlRInit(SR_TO_INT(a));
+    a=aa;
+  }
   if (SR_HDL(b) & SR_INT)
   {
     bb=nlRInit(SR_TO_INT(b));
@@ -766,7 +770,17 @@ number   nlExactDiv(number a, number b, const coeffs r)
   mpz_init(u->z);
   /* u=a/b */
   u->s = 3;
+  assume(a->s==3);
+  assume(b->s==3);
   mpz_divexact(u->z,a->z,b->z);
+  if (aa!=NULL)
+  {
+    mpz_clear(aa->z);
+#if defined(LDEBUG)
+    aa->debug=654324;
+#endif
+    FREE_RNUMBER(aa); // omFreeBin((void *)aa, rnumber_bin);
+  }
   if (bb!=NULL)
   {
     mpz_clear(bb->z);
@@ -950,9 +964,10 @@ int nlDivComp(number a, number b, const coeffs r)
   return 0;
 }
 
-number  nlGetUnit (number n, const coeffs r)
+number  nlGetUnit (number n, const coeffs cf)
 {
-  return INT_TO_SR(1);
+  if (nlGreaterZero(n,cf)) return INT_TO_SR(1);
+  else                     return INT_TO_SR(-1);
 }
 
 coeffs nlQuot1(number c, const coeffs r)
@@ -967,13 +982,13 @@ coeffs nlQuot1(number c, const coeffs r)
   #ifdef HAVE_RINGS
   else
   {
-    mpz_ptr dummy;
-    dummy = (mpz_ptr) omAlloc(sizeof(mpz_t));
+    mpz_t dummy;
     mpz_init_set_ui(dummy, ch);
     ZnmInfo info;
     info.base = dummy;
     info.exp = (unsigned long) 1;
     rr = nInitChar(n_Zn, (void*)&info);
+    mpz_clear(dummy);
   }
   #endif
   return(rr);
@@ -1401,8 +1416,8 @@ number nlNormalizeHelper(number a, number b, const coeffs r)
   if (mpz_cmp_si(gcd,1L)!=0)
   {
     mpz_t bt;
-    mpz_init_set(bt,b->n);
-    mpz_divexact(bt,bt,gcd);
+    mpz_init(bt);
+    mpz_divexact(bt,b->n,gcd);
     if (SR_HDL(a) & SR_INT)
       mpz_mul_si(result->z,bt,SR_TO_INT(a));
     else
@@ -1422,7 +1437,7 @@ number nlNormalizeHelper(number a, number b, const coeffs r)
 
 // Map q \in QQ or ZZ \to Zp or an extension of it
 // src = Q or Z, dst = Zp (or an extension of Zp)
-number nlModP(number q, const coeffs Q, const coeffs Zp)
+number nlModP(number q, const coeffs /*Q*/, const coeffs Zp)
 {
   const int p = n_GetChar(Zp);
   assume( p > 0 );
@@ -1464,21 +1479,21 @@ number nlModP(number q, const coeffs Q, const coeffs Zp)
 /*2
 * convert number i (from Q) to GMP and warn if denom != 1
 */
-void nlGMP(number &i, number n, const coeffs r)
+void nlGMP(number &i, mpz_t n, const coeffs r)
 {
   // Hier brauche ich einfach die GMP Zahl
   nlTest(i, r);
   nlNormalize(i, r);
   if (SR_HDL(i) & SR_INT)
   {
-    mpz_set_si((mpz_ptr) n, SR_TO_INT(i));
+    mpz_set_si(n, SR_TO_INT(i));
     return;
   }
   if (i->s!=3)
   {
      WarnS("Omitted denominator during coefficient mapping !");
   }
-  mpz_set((mpz_ptr) n, i->z);
+  mpz_set(n, i->z);
 }
 #endif
 
@@ -1565,8 +1580,8 @@ BOOLEAN _nlEqual_aNoImm_OR_bNoImm(number a, number b)
     if ((((long)a) < 0L) && (!mpz_isNeg(b->z)))
       return FALSE;
     mpz_t  bb;
-    mpz_init_set(bb,b->n);
-    mpz_mul_si(bb,bb,(long)SR_TO_INT(a));
+    mpz_init(bb);
+    mpz_mul_si(bb,b->n,(long)SR_TO_INT(a));
     bo=(mpz_cmp(bb,b->z)==0);
     mpz_clear(bb);
     return bo;
@@ -1704,8 +1719,8 @@ number _nlAdd_aNoImm_OR_bNoImm(number a, number b)
           return INT_TO_SR(1);
         }
         mpz_init_set(u->n,b->n);
+        u->s = 0;
         if (GCD_NORM_COND(b,u)) { nlNormalize_Gcd(u); }
-        else u->s = 0;
         break;
       }
       case 3:
@@ -1755,8 +1770,8 @@ number _nlAdd_aNoImm_OR_bNoImm(number a, number b)
                FREE_RNUMBER(u);
                return INT_TO_SR(1);
             }
+            u->s = 0;
             if (GCD_NORM_COND(b,u)) { nlNormalize_Gcd(u); }
-            else u->s = 0;
             break;
           }
           case 3: /* a:1 b:3 */
@@ -1776,8 +1791,8 @@ number _nlAdd_aNoImm_OR_bNoImm(number a, number b)
               return INT_TO_SR(1);
             }
             mpz_init_set(u->n,a->n);
+            u->s = 0;
             if (GCD_NORM_COND(a,u)) { nlNormalize_Gcd(u); }
-            else u->s = 0;
             break;
           }
         } /*switch (b->s) */
@@ -1805,8 +1820,8 @@ number _nlAdd_aNoImm_OR_bNoImm(number a, number b)
               return INT_TO_SR(1);
             }
             mpz_init_set(u->n,b->n);
+            u->s = 0;
             if (GCD_NORM_COND(b,u)) { nlNormalize_Gcd(u); }
-            else u->s = 0;
             break;
           }
           case 3:
@@ -1874,8 +1889,9 @@ void _nlInpAdd_aNoImm_OR_bNoImm(number &a, number b)
         mpz_clear(x);
         // result cannot be 0, if coeffs are normalized
         mpz_init_set(u->n,b->n);
+        u->s=0;
         if (GCD_NORM_COND(b,u)) { nlNormalize_Gcd(u); }
-        else { u->s = 0; u=nlShort1(u); }
+        else { u=nlShort1(u); }
         break;
       }
       case 3:
@@ -1914,8 +1930,9 @@ void _nlInpAdd_aNoImm_OR_bNoImm(number &a, number b)
             mpz_clear(x);
             mpz_clear(y);
             mpz_mul(a->n,a->n,b->n);
+            a->s=0;
             if (GCD_NORM_COND(b,a)) { nlNormalize_Gcd(a); }
-            else { a->s = 0;a=nlShort1(a);}
+            else { a=nlShort1(a);}
             break;
           }
           case 3: /* a:1 b:3 */
@@ -1925,8 +1942,9 @@ void _nlInpAdd_aNoImm_OR_bNoImm(number &a, number b)
             mpz_mul(x,b->z,a->n);
             mpz_add(a->z,a->z,x);
             mpz_clear(x);
+            a->s=0;
             if (GCD_NORM_COND(b,a)) { nlNormalize_Gcd(a); }
-            else { a->s = 0; a=nlShort1(a);}
+            else { a=nlShort1(a);}
             break;
           }
         } /*switch (b->s) */
@@ -1945,8 +1963,9 @@ void _nlInpAdd_aNoImm_OR_bNoImm(number &a, number b)
             mpz_add(a->z,b->z,x);
             mpz_clear(x);
             mpz_init_set(a->n,b->n);
+            a->s=0;
             if (GCD_NORM_COND(b,a)) { nlNormalize_Gcd(a); }
-            else { a->s = 0; a=nlShort1(a);}
+            else { a=nlShort1(a);}
             break;
           }
           case 3:
@@ -1995,8 +2014,8 @@ number _nlSub_aNoImm_OR_bNoImm(number a, number b)
           return INT_TO_SR(1);
         }
         mpz_init_set(u->n,b->n);
+        u->s=0;
         if (GCD_NORM_COND(b,u)) { nlNormalize_Gcd(u); }
-        else u->s = 0;
         break;
       }
       case 3:
@@ -2042,8 +2061,8 @@ number _nlSub_aNoImm_OR_bNoImm(number a, number b)
           return INT_TO_SR(1);
         }
         mpz_init_set(u->n,a->n);
+        u->s=0;
         if (GCD_NORM_COND(a,u)) { nlNormalize_Gcd(u); }
-        else u->s = 0;
         break;
       }
       case 3:
@@ -2098,8 +2117,8 @@ number _nlSub_aNoImm_OR_bNoImm(number a, number b)
               FREE_RNUMBER(u);
               return INT_TO_SR(1);
             }
+            u->s=0;
             if (GCD_NORM_COND(a,u)) { nlNormalize_Gcd(u); }
-            else u->s = 0;
             break;
           }
           case 3: /* a:1, b:3 */
@@ -2122,8 +2141,8 @@ number _nlSub_aNoImm_OR_bNoImm(number a, number b)
               return INT_TO_SR(1);
             }
             mpz_init_set(u->n,a->n);
+            u->s=0;
             if (GCD_NORM_COND(a,u)) { nlNormalize_Gcd(u); }
-            else u->s = 0;
             break;
           }
         }
@@ -2154,8 +2173,8 @@ number _nlSub_aNoImm_OR_bNoImm(number a, number b)
               return INT_TO_SR(1);
             }
             mpz_init_set(u->n,b->n);
+            u->s=0;
             if (GCD_NORM_COND(b,u)) { nlNormalize_Gcd(u); }
-            else u->s = 0;
             break;
           }
           case 3: /* a:3 , b:3 */
@@ -2295,7 +2314,7 @@ number _nlMult_aNoImm_OR_bNoImm(number a, number b)
 /*2
 * copy a to b for mapping
 */
-number nlCopyMap(number a, const coeffs src, const coeffs dst)
+number nlCopyMap(number a, const coeffs /*src*/, const coeffs /*dst*/)
 {
   if ((SR_HDL(a) & SR_INT)||(a==NULL))
   {
@@ -2304,7 +2323,7 @@ number nlCopyMap(number a, const coeffs src, const coeffs dst)
   return _nlCopy_NoImm(a);
 }
 
-nMapFunc nlSetMap(const coeffs src, const coeffs dst)
+nMapFunc nlSetMap(const coeffs src, const coeffs /*dst*/)
 {
   if (src->rep==n_rep_gap_rat)  /*Q, coeffs_BIGINT */
   {
@@ -2457,7 +2476,7 @@ LINLINE BOOLEAN nlIsZero (number a, const coeffs)
   }
   return FALSE;
   #else
-  return (a==INT_TO_SR(0));
+  return (a==NULL)|| (a==INT_TO_SR(0));
   #endif
 }
 
@@ -2739,7 +2758,7 @@ number nlQuotRem (number a, number b, number * r, const coeffs R)
       *r = INT_TO_SR(rr);
     if (SR_TO_INT(b)<0)
     {
-      mpz_mul_si(qq, qq, -1);
+      mpz_neg(qq, qq);
     }
     return nlInitMPZ(qq,R);
   }
@@ -2910,9 +2929,9 @@ number nlExtGcd(number a, number b, number *s, number *t, const coeffs)
 void    nlCoeffWrite  (const coeffs r, BOOLEAN /*details*/)
 {
   if (r->is_field)
-  PrintS("//   characteristic : 0\n");
+  PrintS("QQ");
   else
-  PrintS("//   coeff. ring is : Integers\n");
+  PrintS("ZZ");
 }
 
 int n_SwitchChinRem=0;
@@ -2937,24 +2956,28 @@ number   nlChineseRemainderSym(number *x, number *q,int rl, BOOLEAN sym, CFArray
   if (sym)
   {
     number p=CF->convFactoryNSingN(qnew,CF);
-    number p2=nlIntDiv(p,nlInit(2, CF),CF);
-    if (nlGreater(n,p2,CF))
+    number p2;
+    if (getCoeffType(CF) == n_Q) p2=nlIntDiv(p,nlInit(2, CF),CF);
+    else                         p2=CF->cfDiv(p,CF->cfInit(2, CF),CF);
+    if (CF->cfGreater(n,p2,CF))
     {
-       number n2=nlSub(n,p,CF);
-       nlDelete(&n,CF);
+       number n2=CF->cfSub(n,p,CF);
+       CF->cfDelete(&n,CF);
        n=n2;
     }
-    nlDelete(&p2,CF);
-    nlDelete(&p,CF);
+    CF->cfDelete(&p2,CF);
+    CF->cfDelete(&p,CF);
   }
-  nlNormalize(n,CF);
+  CF->cfNormalize(n,CF);
   return n;
 }
-number   nlChineseRemainder(number *x, number *q,int rl, const coeffs C)
+#if 0
+number nlChineseRemainder(number *x, number *q,int rl, const coeffs C)
 {
   CFArray inv(rl);
   return nlChineseRemainderSym(x,q,rl,TRUE,inv,C);
 }
+#endif
 
 static void nlClearContent(ICoeffsEnumerator& numberCollectionEnumerator, number& c, const coeffs cf)
 {
@@ -3150,8 +3173,8 @@ char * nlCoeffName(const coeffs r)
 static char* nlCoeffString(const coeffs r)
 {
   //return omStrDup(nlCoeffName(r));
-  if (r->cfDiv==nlDiv) return omStrDup("0");
-  else                 return omStrDup("integer");
+  if (r->cfDiv==nlDiv) return omStrDup("QQ");
+  else                 return omStrDup("ZZ");
 }
 
 static void nlWriteFd(number n,FILE* f, const coeffs)
@@ -3303,8 +3326,6 @@ BOOLEAN nlInitChar(coeffs r, void*p)
   r->is_domain=TRUE;
   r->rep=n_rep_gap_rat;
 
-  //const int ch = (int)(long)(p);
-
   r->nCoeffIsEqual=nlCoeffIsEqual;
   //r->cfKillChar = ndKillChar; /* dummy */
   r->cfCoeffString=nlCoeffString;
@@ -3316,6 +3337,7 @@ BOOLEAN nlInitChar(coeffs r, void*p)
   r->cfMult  = nlMult;
   r->cfSub   = nlSub;
   r->cfAdd   = nlAdd;
+  r->cfExactDiv= nlExactDiv;
   if (p==NULL) /* Q */
   {
     r->is_field=TRUE;
@@ -3338,7 +3360,6 @@ BOOLEAN nlInitChar(coeffs r, void*p)
     r->cfXExtGcd=nlXExtGcd;
     r->cfQuotRem=nlQuotRem;
   }
-  r->cfExactDiv= nlExactDiv;
   r->cfInit = nlInit;
   r->cfSize  = nlSize;
   r->cfInt  = nlInt;
