@@ -47,26 +47,7 @@ const char *  npRead  (const char *s, number *a,const coeffs r);
 BOOLEAN npDBTest      (number a, const char *f, const int l, const coeffs r);
 #endif
 
-//int     npGetChar();
-
 nMapFunc npSetMap(const coeffs src, const coeffs dst);
-number  npMapP(number from, const coeffs src, const coeffs r);
-
-// extern int npGen; // obsolete
-
-// int npGen=0;
-
-/*-------specials for spolys, do NOT use otherwise--------------------------*/
-/* for npMultM, npSubM, npNegM, npEqualM : */
-#ifdef HAVE_DIV_MOD
-extern unsigned short *npInvTable;
-#else
-#ifndef HAVE_MULT_MOD
-extern long npPminus1M;
-extern unsigned short *npExpTable;
-extern unsigned short *npLogTable;
-#endif
-#endif
 
 #ifdef NV_OPS
 #pragma GCC diagnostic ignored "-Wlong-long"
@@ -88,9 +69,6 @@ number  nvInvers      (number c, const coeffs r);
 //void    nvPower       (number a, int i, number * result, const coeffs r);
 #endif
 
-
-
-
 BOOLEAN npGreaterZero (number k, const coeffs r)
 {
   n_Test(k, r);
@@ -130,7 +108,6 @@ number npInit (long i, const coeffs r)
   number c = (number)ii;
   n_Test(c, r);
   return c;
-
 }
 
 
@@ -303,7 +280,6 @@ number  npInvers (number c, const coeffs r)
 
   n_Test(d, r);
   return d;
-
 }
 
 number npNeg (number c, const coeffs r)
@@ -374,7 +350,6 @@ void npPower (number a, int i, number * result, const coeffs r)
 
 static const char* npEati(const char *s, int *i, const coeffs r)
 {
-
   if (((*s) >= '0') && ((*s) <= '9'))
   {
     unsigned long ii=0L;
@@ -467,24 +442,24 @@ number npConvFactoryNSingN( const CanonicalForm n, const coeffs r)
   }
 }
 
-static char* npCoeffString(const coeffs r)
+static char* npCoeffName(const coeffs cf)
 {
-#ifdef SINGULAR_4_1
-  char *s=(char*)omAlloc(14);
-  snprintf(s,14,"ZZ/%d",r->ch);
-#else
-  char *s=(char*)omAlloc(11);
-  snprintf(s,11,"%d",r->ch);
-#endif
-  return s;
+  static char npCoeffName_buf[15];
+  snprintf(npCoeffName_buf,14,"ZZ/%d",cf->ch);
+  return npCoeffName_buf;
 }
 
-static void npWriteFd(number n, FILE* f, const coeffs r)
+static char* npCoeffString(const coeffs cf)
+{
+  return omStrDup(npCoeffName(cf));
+}
+
+static void npWriteFd(number n, FILE* f, const coeffs)
 {
   fprintf(f,"%d ",(int)(long)n);
 }
 
-static number npReadFd(s_buff f, const coeffs r)
+static number npReadFd(s_buff f, const coeffs)
 {
   // read int
   int dd;
@@ -517,6 +492,8 @@ BOOLEAN npInitChar(coeffs r, void* p)
   r->cfKillChar=npKillChar;
   r->nCoeffIsEqual=npCoeffsEqual;
   r->cfCoeffString=npCoeffString;
+  r->cfCoeffName=npCoeffName;
+  r->cfCoeffWrite=npCoeffWrite;
 
   r->cfMult  = npMult;
   r->cfSub   = npSub;
@@ -565,7 +542,6 @@ BOOLEAN npInitChar(coeffs r, void* p)
     //r->cfPower= nvPower;
   }
 #endif
-  r->cfCoeffWrite=npCoeffWrite;
 #ifdef LDEBUG
   // debug stuff
   r->cfDBTest=npDBTest;
@@ -583,7 +559,6 @@ BOOLEAN npInitChar(coeffs r, void* p)
   // the variables:
   r->nNULL = (number)0;
   r->type = n_Zp;
-  r->ch = c;
   r->has_simple_Alloc=TRUE;
   r->has_simple_Inverse=TRUE;
 
@@ -642,7 +617,7 @@ BOOLEAN npDBTest (number a, const char *f, const int l, const coeffs r)
 }
 #endif
 
-number npMapP(number from, const coeffs src, const coeffs dst_r)
+static number npMapP(number from, const coeffs src, const coeffs dst_r)
 {
   long i = (long)from;
   if (i>src->ch/2)
@@ -730,7 +705,7 @@ static number npMapLongR(number from, const coeffs /*src*/, const coeffs dst_r)
 /*2
 * convert from a GMP integer
 */
-number npMapGMP(number from, const coeffs /*src*/, const coeffs dst)
+static number npMapGMP(number from, const coeffs /*src*/, const coeffs dst)
 {
   mpz_ptr erg = (mpz_ptr) omAlloc(sizeof(mpz_t)); // evtl. spaeter mit bin
   mpz_init(erg);
@@ -743,7 +718,7 @@ number npMapGMP(number from, const coeffs /*src*/, const coeffs dst)
   return (number) r;
 }
 
-number npMapZ(number from, const coeffs src, const coeffs dst)
+static number npMapZ(number from, const coeffs src, const coeffs dst)
 {
   if (SR_HDL(from) & SR_INT)
   {
@@ -756,14 +731,14 @@ number npMapZ(number from, const coeffs src, const coeffs dst)
 /*2
 * convert from an machine long
 */
-number npMapMachineInt(number from, const coeffs /*src*/,const coeffs dst)
+static number npMapMachineInt(number from, const coeffs /*src*/,const coeffs dst)
 {
   long i = (long) (((unsigned long) from) % dst->ch);
   return (number) i;
 }
 #endif
 
-number npMapCanonicalForm (number a, const coeffs /*src*/, const coeffs dst)
+static number npMapCanonicalForm (number a, const coeffs /*src*/, const coeffs dst)
 {
   setCharacteristic (dst ->ch);
   CanonicalForm f= CanonicalForm ((InternalCF*)(a));
@@ -925,6 +900,6 @@ void nvPower (number a, int i, number * result, const coeffs r)
 
 void    npCoeffWrite  (const coeffs r, BOOLEAN /*details*/)
 {
-  Print("//   characteristic : %d\n",r->ch);
+  Print("ZZ/%d",r->ch);
 }
 
