@@ -71,6 +71,52 @@ typedef enum
 }
 ro_typ;
 
+/// order stuff
+typedef enum rRingOrder_t
+{
+  ringorder_no = 0,
+  ringorder_a,
+  ringorder_a64, ///< for int64 weights
+  ringorder_c,
+  ringorder_C,
+  ringorder_M,
+  ringorder_S, ///< S?
+  ringorder_s, ///< s?
+  ringorder_lp,
+  ringorder_dp,
+  ringorder_rp,
+  ringorder_Dp,
+  ringorder_wp,
+  ringorder_Wp,
+  ringorder_ls,
+  ringorder_ds,
+  ringorder_Ds,
+  ringorder_ws,
+  ringorder_Ws,
+  ringorder_am,
+  ringorder_L,
+  // the following are only used internally
+  ringorder_aa, ///< for idElimination, like a, except pFDeg, pWeigths ignore it
+  ringorder_rs, ///< opposite of ls
+  ringorder_IS, ///< Induced (Schreyer) ordering
+  ringorder_unspec
+} rRingOrder_t;
+
+typedef enum rOrderType_t
+{
+  rOrderType_General = 0, ///< non-simple ordering as specified by currRing
+  rOrderType_CompExp,     ///< simple ordering, component has priority
+  rOrderType_ExpComp,     ///< simple ordering, exponent vector has priority
+                          ///< component not compatible with exp-vector order
+  rOrderType_Exp,         ///< simple ordering, exponent vector has priority
+                          ///< component is compatible with exp-vector order
+  rOrderType_Syz,         ///< syzygy ordering
+  rOrderType_Schreyer,    ///< Schreyer ordering
+  rOrderType_Syz2dpc,     ///< syzcomp2dpc
+  rOrderType_ExpNoComp    ///< simple ordering, differences in component are
+                          ///< not considered
+} rOrderType_t;
+
 // ordering is a degree ordering
 struct sro_dp
 {
@@ -212,7 +258,7 @@ struct ip_sring
 // general ordering: pointer/structs, long, int, short, BOOLEAN/char/enum
 // general defining procedures: rInit, rComplete, interpreter, ??
   idhdl      idroot; /* local objects , interpreter*/
-  int*       order;  /* array of orderings, rInit/rSleftvOrdering2Ordering */
+  rRingOrder_t* order;  /* array of orderings, rInit/rSleftvOrdering2Ordering */
   int*       block0; /* starting pos., rInit/rSleftvOrdering2Ordering*/
   int*       block1; /* ending pos., rInit/rSleftvOrdering2Ordering*/
 //  char**     parameter; /* names of parameters, rInit */
@@ -282,7 +328,7 @@ struct ip_sring
   BOOLEAN   MixedOrder; // TRUE for global/local mixed orderings, FALSE otherwise
   BOOLEAN   pLexOrder; /* TRUE if the monomial ordering is not compatible with pFDeg */
 
-  BOOLEAN   ComponentOrder; // ???
+  BOOLEAN   ComponentOrder; // 1 if ringorder_c, -1 for ringorder_C,ringorder_S,ringorder_s
 
   // what follows below here should be set by rComplete, _only_
   // contains component, but no weight fields in E */
@@ -301,9 +347,6 @@ struct ip_sring
   /* if >= 0, long vars in exp vector are consecutive and start there
      if <  0, long vars in exp vector are not consecutive */
   short     VarL_LowIndex;
-  // number of exponents in r->VarL_Offset[0]
-  // is minimal number of exponents in a long var
-  short     MinExpPerLong;
 
   short     NegWeightL_Size;
   /* array of size VarL_Size,
@@ -343,9 +386,9 @@ struct ip_sring
 /////// void   rChangeCurrRing(ring r);
 
 ring   rDefault(int ch, int N, char **n);
-ring   rDefault(const coeffs cf, int N, char **n);
-ring   rDefault(int ch, int N, char **n,int ord_size, int *ord, int *block0, int *block1, int **wvhdl=NULL);
-ring   rDefault(const coeffs cf, int N, char **n,int ord_size, int *ord, int *block0, int *block1, int **wvhdl=NULL);
+ring   rDefault(const coeffs cf, int N, char **n, const rRingOrder_t o=ringorder_lp);
+ring   rDefault(int ch, int N, char **n,int ord_size, rRingOrder_t *ord, int *block0, int *block1, int **wvhdl=NULL);
+ring   rDefault(const coeffs cf, int N, char **n,int ord_size, rRingOrder_t *ord, int *block0, int *block1, int **wvhdl=NULL);
 
 // #define rIsRingVar(A) r_IsRingVar(A,currRing)
 int    r_IsRingVar(const char *n, char**names, int N);
@@ -388,7 +431,7 @@ void rGetSComps(int** currComponents, long** currShiftedComponents, int *length,
 
 
 const char * rSimpleOrdStr(int ord);
-int rOrderName(char * ordername);
+rRingOrder_t rOrderName(char * ordername);
 char * rOrdStr(ring r);
 char * rVarStr(ring r);
 char * rCharStr(ring r);
@@ -458,6 +501,9 @@ static inline BOOLEAN rField_is_Zp(const ring r, int p)
 static inline BOOLEAN rField_is_Q(const ring r)
 { assume(r != NULL); assume(r->cf != NULL); return nCoeff_is_Q(r->cf); }
 
+static inline BOOLEAN rField_is_Z(const ring r)
+{ assume(r != NULL); assume(r->cf != NULL); return nCoeff_is_Z(r->cf); }
+
 static inline BOOLEAN rField_is_numeric(const ring r) /* R, long R, long C */
 { assume(r != NULL); assume(r->cf != NULL); return nCoeff_is_numeric(r->cf); }
 
@@ -662,52 +708,6 @@ static inline BOOLEAN rMinpolyIsNULL(const ring r)
 
 
 
-/// order stuff
-typedef enum rRingOrder_t
-{
-  ringorder_no = 0,
-  ringorder_a,
-  ringorder_a64, ///< for int64 weights
-  ringorder_c,
-  ringorder_C,
-  ringorder_M,
-  ringorder_S, ///< S?
-  ringorder_s, ///< s?
-  ringorder_lp,
-  ringorder_dp,
-  ringorder_rp,
-  ringorder_Dp,
-  ringorder_wp,
-  ringorder_Wp,
-  ringorder_ls,
-  ringorder_ds,
-  ringorder_Ds,
-  ringorder_ws,
-  ringorder_Ws,
-  ringorder_am,
-  ringorder_L,
-  // the following are only used internally
-  ringorder_aa, ///< for idElimination, like a, except pFDeg, pWeigths ignore it
-  ringorder_rs, ///< opposite of ls
-  ringorder_IS, ///< Induced (Schreyer) ordering
-  ringorder_unspec
-} rRingOrder_t;
-
-typedef enum rOrderType_t
-{
-  rOrderType_General = 0, ///< non-simple ordering as specified by currRing
-  rOrderType_CompExp,     ///< simple ordering, component has priority
-  rOrderType_ExpComp,     ///< simple ordering, exponent vector has priority
-                          ///< component not compatible with exp-vector order
-  rOrderType_Exp,         ///< simple ordering, exponent vector has priority
-                          ///< component is compatible with exp-vector order
-  rOrderType_Syz,         ///< syzygy ordering
-  rOrderType_Schreyer,    ///< Schreyer ordering
-  rOrderType_Syz2dpc,     ///< syzcomp2dpc
-  rOrderType_ExpNoComp    ///< simple ordering, differences in component are
-                          ///< not considered
-} rOrderType_t;
-
 static inline BOOLEAN rIsSyzIndexRing(const ring r)
 { assume(r != NULL); assume(r->cf != NULL); return r->order[0] == ringorder_s;}
 
@@ -720,6 +720,7 @@ void   rSetSyzComp(int k, const ring r);
 ring   rAssure_HasComp(const ring r);
 ring   rAssure_SyzOrder(const ring r, BOOLEAN complete);
 ring   rAssure_SyzComp(const ring r, BOOLEAN complete = TRUE);
+ring   rAssure_InducedSchreyerOrdering(const ring r, BOOLEAN complete = TRUE, int sgn = 1);
 
 ring   rAssure_dp_S(const ring r);
 ring   rAssure_dp_C(const ring r);
@@ -730,8 +731,9 @@ ring   rAssure_c_dp(const ring r);
 ring   rAssure_CompLastBlock(const ring r, BOOLEAN complete = TRUE);
 
 /// makes sure that c/C ordering is last ordering and SyzIndex is first
-ring   rAssure_SyzComp_CompLastBlock(const ring r, BOOLEAN complete = TRUE);
-ring   rAssure_TDeg(const ring r, int start_var, int end_var, int &pos);
+ring   rAssure_SyzComp_CompLastBlock(const ring r);
+ring   rAssure_TDeg(const ring r, int &pos);
+BOOLEAN rHasTDeg(const ring r);
 
 /// return the max-comonent wchich has syzIndex i
 /// Assume: i<= syzIndex_limit
@@ -753,9 +755,6 @@ inline BOOLEAN rHasMixedOrdering(const ring r) { return (r->MixedOrder); }
 // #define rHasGlobalOrdering(R) ((R)->OrdSgn==1)
 // #define rHasLocalOrMixedOrdering(R) ((R)->OrdSgn==-1)
 
-#define rHasGlobalOrdering_currRing() rHasGlobalOrdering(currRing)
-#define rHasLocalOrMixedOrdering_currRing() rHasLocalOrMixedOrdering(currRing)
-
 BOOLEAN rOrd_is_Totaldegree_Ordering(const ring r);
 
 /// return TRUE if p_SetComp requires p_Setm
@@ -817,6 +816,7 @@ BOOLEAN rSetISReference(const ring r, const ideal F, const int i = 0, const int
 
 /// return the position of the p^th IS block order block in r->typ[]...
 int rGetISPos(const int p, const ring r);
+void pISUpdateComponents(ideal F, const intvec *const V, const int MIN, const ring r);
 
 BOOLEAN rCheckIV(const intvec *iv);
 int rTypeOfMatrixOrder(const intvec *order);
